@@ -5,7 +5,7 @@ from random import uniform as u
 
 from vedo import Plotter, procrustesAlignment, Points
 
-vp = Plotter(shape=[1, 2], axes=2, sharecam=0)
+plt = Plotter(shape=[1, 2], axes=2, sharecam=0)
 
 N = 15  # number of points
 x = 1.0  # add some randomness
@@ -18,7 +18,7 @@ vpts1 = Points(pts1, c="r").legend("set1")
 vpts2 = Points(pts2, c="g").legend("set2")
 vpts3 = Points(pts3, c="b").legend("set3")
 
-vp.show(vpts1, vpts2, vpts3, __doc__, at=0)
+plt.show(vpts1, vpts2, vpts3, __doc__, at=0)
 
 # find best alignment among the n sets of Points,
 # return an Assembly object formed by the aligned sets
@@ -26,4 +26,4 @@ aligned = procrustesAlignment([vpts1, vpts2, vpts3])
 
 # print(aligned.info['transform'])
 
-vp.show(aligned, at=1, interactive=1).close()
+plt.show(aligned, at=1).interactive().close()
