@@ -1,10 +1,10 @@
 """Add a square button with N possible internal states
 to a rendering window that calls an external function"""
-from vedo import *
+from vedo import Plotter, Mesh, dataurl, printc
 
-vp = Plotter()
+plt = Plotter(axes=11)
 
-mesh = vp.load(dataurl+"magnolia.vtk").c("v").flat()
+mesh = Mesh(dataurl+"magnolia.vtk").c("v").flat()
 
 # add a button to the current renderer (e.i. nr1)
 def buttonfunc():
@@ -12,7 +12,7 @@ def buttonfunc():
     bu.switch()                 # change to next status
     printc(bu.status(), box="_", dim=True)
 
-bu = vp.addButton(
+bu = plt.addButton(
     buttonfunc,
     pos=(0.7, 0.05),  # x,y fraction from bottom left corner
     states=["click to hide", "click to show"],
@@ -24,4 +24,4 @@ bu = vp.addButton(
     italic=False,
 )
 
-vp.show(mesh, __doc__, axes=11).close()
+plt.show(mesh, __doc__).close()
