@@ -3,10 +3,10 @@ Example usage of cmap() to assign a color to each mesh vertex
 by looking it up in matplotlib database of colormaps
 """
 print(__doc__)
-from vedo import Plotter, dataurl
+from vedo import Plotter, Mesh, dataurl
 
 # these are the some matplotlib color maps
-mapkeys = [
+maps = [
     "afmhot",
     "binary",
     "bone",
@@ -21,15 +21,13 @@ mapkeys = [
     "winter",
 ]
 
-vp = Plotter(N=len(mapkeys))
-vp.legendSize = 0.4
-
-mug = vp.load(dataurl+"mug.ply")
-
+mug = Mesh(dataurl+"mug.ply")
 scalars = mug.points()[:, 1]  # let y-coord be the scalar
 
-for i, key in enumerate(mapkeys):  # for each available color map name
-    imug = mug.clone(deep=False).cmap(key, scalars, n=5)
-    vp.show(imug, key, at=i)
+plt = Plotter(N=len(maps))
 
-vp.show(interactive=True).close()
+for i, key in enumerate(maps):  # for each available color map name
+    imug = mug.clone(deep=False).cmap(key, scalars, n=5)
+    plt.show(imug, key, at=i)
+
+plt.show().interactive().close()
