@@ -2,17 +2,17 @@
 Show a cube for each available texture name.
 Any jpg file can be used as texture.
 """
-from vedo import settings, Plotter, Cube, Text2D
-from vedo.settings import textures, textures_path
+from vedo import settings, Plotter, Cube
+from vedo import textures, textures_path
 
 print(__doc__)
-print('textures_path:', settings.textures_path)
-print('textures:', settings.textures)
+print('textures_path:', textures_path)
+print('textures:', textures)
 
 settings.immediateRendering = False
-plt = Plotter(N=len(settings.textures), axes=0)
+plt = Plotter(N=len(textures), axes=0)
 
-for i, name in enumerate(settings.textures):
+for i, name in enumerate(textures):
     if i>30: break
     cb = Cube().texture(name)
     plt.show(cb, name, at=i, azimuth=1)
