@@ -7,17 +7,16 @@ from vedo.applications import Animation
 
 s = load(dataurl+"bunny.obj").subdivide().normalize()
 
-vp = Animation()
-vp.timeResolution = 0.02  # secs
+plt = Animation()
+plt.timeResolution = 0.02  # secs
 
-vp.switchOn(s)
+plt.switchOn(s)
 
 # no need to repeat t=1, duration=3 in changeLighting and changeColor
-vp.meshErode(corner=0, t=1, duration=3).changeLighting("glossy").changeColor("v")
+plt.meshErode(corner=0, t=1, duration=3).changeLighting("glossy").changeColor("v")
 
 cam1 = orientedCamera(backoffVector=(0, 0, -1), backoff=8)
 cam2 = orientedCamera(backoffVector=(1, 1,  1), backoff=8)
 
-vp.moveCamera(cam1, cam2, t=0, duration=4)
-
-vp.play()
+plt.moveCamera(cam1, cam2, t=0, duration=4)
+plt.play()
