@@ -64,20 +64,19 @@ def morph(clm1, clm2, t, lmax):
     return pts
 
 
-vp = Plotter(shape=[2, 2], axes=3, interactive=0)
+plt = Plotter(shape=[2, 2], axes=3, interactive=0)
 
 shape1 = Sphere(alpha=0.2)
-shape2 = vp.load(dataurl + "icosahedron.vtk").normalize().lineWidth(1)
+shape2 = plt.load(dataurl + "icosahedron.vtk").normalize().lineWidth(1)
 
 agrid1, actorpts1 = makeGrid(shape1, N)
 
-vp.show(shape1, actorpts1, at=0)
+plt.show(shape1, actorpts1, at=0)
 
 agrid2, actorpts2 = makeGrid(shape2, N)
-vp.show(shape2, actorpts2, at=1)
+plt.show(shape2, actorpts2, at=1)
 
-vp.camera.Zoom(1.2)
-vp.interactive = False
+plt.camera.Zoom(1.2)
 
 clm1 = pyshtools.SHGrid.from_array(agrid1).expand()
 clm2 = pyshtools.SHGrid.from_array(agrid2).expand()
@@ -88,8 +87,8 @@ for t in np.arange(0, 1, 0.005):
     act21 = Points(morph(clm2, clm1, t, lmax), c="r", r=4)
     act12 = Points(morph(clm1, clm2, t, lmax), c="g", r=4)
 
-    vp.show(act21, at=2, resetcam=0)
-    vp.show(act12, at=3)
-    vp.camera.Azimuth(2)
+    plt.show(act21, at=2, resetcam=0)
+    plt.show(act12, at=3)
+    plt.camera.Azimuth(2)
 
-vp.show(interactive=1)
+plt.show().interactive().close()
