@@ -1,14 +1,16 @@
-"""Simulation of a block connected 
+"""Simulation of a block connected
 to a spring in a viscous medium"""
 from vedo import *
 
+settings.allowInteraction = True  # allow mouse interaction while playing
+
 plt = Plotter(interactive=0, axes=0)
 
-L = 0.1  # spring x position at rest
+L = 0.1    # spring x position at rest
 x0 = 0.85  # initial x-coordinate of the block
-k = 25  # spring constant
-m = 20  # block mass
-b = 0.5  # viscosity friction (proportional to velocity)
+k = 25     # spring constant
+m = 20     # block mass
+b = 0.5    # viscosity friction (proportional to velocity)
 dt = 0.15  # time step
 
 # initial conditions
@@ -23,8 +25,10 @@ plt += Box(pos=(-0.82, 0.15, 0), length=0.04, width=0.50, height=0.3)  # wall
 
 block = Cube(pos=x, side=0.2, c="tomato")
 block.addTrail(offset=[0, 0.2, 0], lw=2, n=500)
+
 spring = Spring(sx0, x, r=0.06, thickness=0.01)
-plt += [block, spring, Text2D(__doc__)]
+
+plt += [block, spring, __doc__]
 
 pb = ProgressBar(0, 300, c="r")
 for i in pb.range():
@@ -40,4 +44,4 @@ for i in pb.range():
     if plt.escaped: break # if ESC is hit during the loop
     pb.print()
 
-plt.show(interactive=1).close()
+plt.interactive().close()
