@@ -66,4 +66,4 @@ for i, t in enumerate(pb.range()):
         if plt.escaped: break # if ESC is hit during the loop
     pb.print()
 
-interactive().close()
+plt.interactive().close()
