@@ -1,8 +1,8 @@
 import numpy as np
-import vtk
 import vedo
 import vedo.colors as colors
 import vedo.utils as utils
+import vtk
 from deprecated import deprecated
 
 __doc__ = (
@@ -53,8 +53,8 @@ class _DataArrayHelper(object):
             self.actor._mapper.SetScalarModeToUseCellData()
 
         if len(input_array) != n:
-            colors.printc(f'Error in point/cell data: length of input {len(input_array)}'
-                          f' !=  {n} nr. of elements', c='r')
+            vedo.logger.error(f'Error in point/cell data: length of input {len(input_array)}'
+                              f' !=  {n} nr. of elements')
             raise RuntimeError()
 
         input_array = np.ascontiguousarray(input_array)
@@ -95,7 +95,7 @@ class _DataArrayHelper(object):
         if varr:
             varr.SetName(newname)
         else:
-            colors.printc("Cannot rename non existing array:", oldname, "to:", newname, c='r')
+            vedo.logger.warning(f"Cannot rename non existing array {oldname} to {newname}")
 
     def select(self, key):
         if self.association == 0:
@@ -144,6 +144,9 @@ class Base3DProp(object):
         self.transform = None
         self._set2actcam = False # used by mesh.followCamera()
 
+        self.point_locator = None
+        self.cell_locator = None
+
 
     def address(self):
         """
@@ -206,6 +209,9 @@ class Base3DProp(object):
             z=0
         self.SetPosition(x, y, z)
 
+        self.point_locator = None
+        self.cell_locator = None
+
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -224,6 +230,9 @@ class Base3DProp(object):
         else:
             self.SetPosition(p + [dx,dy,dz])
 
+        self.point_locator = None
+        self.cell_locator = None
+
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -239,7 +248,7 @@ class Base3DProp(object):
         p = self.GetPosition()
         if position is None:
             return p[0]
-        self.SetPosition(position, p[1], p[2])
+        self.pos(position, p[1], p[2])
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -251,7 +260,7 @@ class Base3DProp(object):
         p = self.GetPosition()
         if position is None:
             return p[1]
-        self.SetPosition(p[0], position, p[2])
+        self.pos(p[0], position, p[2])
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -263,7 +272,7 @@ class Base3DProp(object):
         p = self.GetPosition()
         if position is None:
             return p[2]
-        self.SetPosition(p[0], p[1], position)
+        self.pos(p[0], p[1], position)
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -307,7 +316,8 @@ class Base3DProp(object):
             angle *= 180.0 / np.pi
         # this vtk method only rotates in the origin of the object:
         self.RotateWXYZ(angle, axis[0], axis[1], axis[2])
-        self.SetPosition(rv)
+        self.pos(rv)
+
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -335,6 +345,9 @@ class Base3DProp(object):
                 self.addShadows()
         else:
             self.RotateX(angle)
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
     def rotateY(self, angle, rad=False, locally=False):
@@ -358,6 +371,9 @@ class Base3DProp(object):
                 self.addShadows()
         else:
             self.RotateY(angle)
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
     def rotateZ(self, angle, rad=False, locally=False):
@@ -381,6 +397,9 @@ class Base3DProp(object):
                 self.addShadows()
         else:
             self.RotateZ(angle)
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
 
@@ -426,6 +445,10 @@ class Base3DProp(object):
         T.RotateWXYZ(np.rad2deg(angle), crossvec)
         T.Translate(pos)
         self.SetUserTransform(T)
+
+        self.point_locator = None
+        self.cell_locator = None
+
         if self.trail:
             self.updateTrail()
         if len(self.shadows) > 0:
@@ -445,6 +468,9 @@ class Base3DProp(object):
             self.SetScale(s)
         else:
             self.SetScale(np.multiply(self.GetScale(), s))
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
 
@@ -474,16 +500,23 @@ class Base3DProp(object):
                 l = Line(p-v, p+v).lw(3).c('red')
                 show(c1.wireframe().lw(3), l, c2, axes=1)
         """
-        if self.transform:
-            tr = self.transform
-        else:
-            T = self.GetMatrix()
-            tr = vtk.vtkTransform()
-            tr.SetMatrix(T)
+        # if self.transform:
+        #     tr = self.transform
+        # else:
+        #     T = self.GetMatrix()
+        #     tr = vtk.vtkTransform()
+        #     tr.SetMatrix(T)
+        # if invert:
+        #     tr = tr.GetInverse()
+        # return tr if self.transform:
+        T = self.GetMatrix()
+        tr = vtk.vtkTransform()
+        tr.SetMatrix(T)
         if invert:
             tr = tr.GetInverse()
         return tr
 
+
     def applyTransform(self, T):
         """
         Transform object position and orientation.
@@ -499,6 +532,9 @@ class Base3DProp(object):
         else:
             self.SetUserTransform(T)
         self.transform = T
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
     def alignToBoundingBox(self, msh, rigid=False):
@@ -553,6 +589,9 @@ class Base3DProp(object):
         lmt.Update()
         self.applyTransform(lmt)
         self.transform = lmt
+
+        self.point_locator = None
+        self.cell_locator = None
         return self
 
 
@@ -749,6 +788,13 @@ class BaseActor(Base3DProp):
             return self._mapper.GetInput()
         return self.GetMapper().GetInput()
 
+    def modified(self):
+        """Use in conjunction with ``tonumpy()`` to update any modifications to the volume array"""
+        sc = self.inputdata().GetPointData().GetScalars()
+        if sc:
+            sc.Modified()
+        self.inputdata().GetPointData().Modified()
+        return self
 
     def N(self):
         """Retrieve number of points. Shortcut for `NPoints()`."""
@@ -798,6 +844,8 @@ class BaseActor(Base3DProp):
             vpts.Modified()
             # reset mesh to identity matrix position/rotation:
             self.PokeMatrix(vtk.vtkMatrix4x4())
+            self.point_locator = None
+            self.cell_locator = None
             return self
 
     def cellCenters(self):
@@ -926,8 +974,8 @@ class BaseActor(Base3DProp):
             elif style=='ambient' : pars = [0.8, 0.1, 0.0,  1, (1,1,1)]
             elif style=='default' : pars = [0.1, 1.0, 0.05, 5, c]
             else:
-                colors.printc("Error in lighting(): Available styles are", c='r')
-                colors.printc("[default,metallic,plastic,shiny,glossy,ambient,off]", c='r')
+                vedo.logger.error("in lighting(): Available styles are")
+                vedo.logger.error("[default, metallic, plastic, shiny, glossy, ambient, off]")
                 raise RuntimeError()
             pr.SetAmbient(pars[0])
             pr.SetDiffuse(pars[1])
@@ -1110,7 +1158,7 @@ class BaseActor(Base3DProp):
             if self.GetScalars():
                 arrname = varr.GetScalars().GetName()
             else:
-                colors.printc('Error in gradient: no scalars found for', on, c='r')
+                vedo.logger.error(f"in gradient: no scalars found for {on}")
                 raise RuntimeError
         gra.SetInputData(self.inputdata())
         gra.SetInputScalars(tp, arrname)
@@ -1146,7 +1194,7 @@ class BaseActor(Base3DProp):
             if self.GetVectors():
                 arrname = varr.GetVectors().GetName()
             else:
-                colors.printc('Error in divergence: no scalars found for', on, c='r')
+                vedo.logger.error(f"in divergence(): no scalars found for {on}")
                 raise RuntimeError
         div.SetInputData(self.inputdata())
         div.SetInputScalars(tp, arrname)
@@ -1182,7 +1230,7 @@ class BaseActor(Base3DProp):
             if self.GetVectors():
                 arrname = varr.GetVectors().GetName()
             else:
-                colors.printc('Error in vortergence: no scalars found for', on, c='r')
+                vedo.logger.error(f"in vorticity(): no scalars found for {on}")
                 raise RuntimeError
         vort.SetInputData(self.inputdata())
         vort.SetInputScalars(tp, arrname)
@@ -1209,17 +1257,13 @@ class BaseActor(Base3DProp):
                      horizontal=False,
                      useAlpha=True,
                      tformat='%-#6.3g',
-    ):
+        ):
         """
         Add a 2D scalar bar for the specified obj.
 
         .. hint:: |mesh_coloring| |mesh_coloring.py|_ |scalarbars.py|_
         """
-        plt = vedo.settings.plotter_instance
-
-        if not hasattr(self, "mapper"):
-            colors.printc("Error in addScalarBar(): input is invalid,", type(self), c='r')
-            return None
+        plt = vedo.plotter_instance
 
         if plt and plt.renderer:
             c = (0.9, 0.9, 0.9)
@@ -1287,7 +1331,7 @@ class BaseActor(Base3DProp):
 
         .. hint:: |scalarbars| |scalarbars.py|_
         """
-        plt = vedo.settings.plotter_instance
+        plt = vedo.plotter_instance
         if plt and c is None:  # automatic black or white
             c = (0.9, 0.9, 0.9)
             if np.sum(vedo.getColor(plt.backgrcol)) > 1.5:
@@ -1468,7 +1512,7 @@ class BaseGrid(BaseActor):
             ctf.AddRGBPoint(vmin, r,g,b) # constant color
             ctf.AddRGBPoint(vmax, r,g,b)
         else:
-            colors.printc("color(): unknown input type:", col, c='r')
+            vedo.logger.warning(f"in color() unknown input type {type(col)}")
 
         if alpha is not None:
             self.alpha(alpha, vmin=vmin, vmax=vmax)
@@ -1582,7 +1626,7 @@ class BaseGrid(BaseActor):
         return a
 
 
-    def legosurface(self, vmin=None, vmax=None, invert=False, cmap='afmhot_r', boundary=True):
+    def legosurface(self, vmin=None, vmax=None, invert=False, boundary=False):
         """
         Represent an object - typically a Volume - as lego blocks (voxels).
         By default colors correspond to the volume's scalar.
@@ -1590,7 +1634,7 @@ class BaseGrid(BaseActor):
 
         :param float vmin: the lower threshold, voxels below this value are not shown.
         :param float vmax: the upper threshold, voxels above this value are not shown.
-        :param str cmap: color mapping of the scalar associated to the voxels.
+        :param bool boundary: controls whether to include cells that are partially inside
 
         |legosurface| |legosurface.py|_
         """
@@ -1620,12 +1664,11 @@ class BaseGrid(BaseActor):
         gf.SetInputData(extract.GetOutput())
         gf.Update()
 
-        a = vedo.mesh.Mesh(gf.GetOutput()).lw(0.1).flat().lighting('ambient')
+        a = vedo.mesh.Mesh(gf.GetOutput()).lw(0.1).flat()#.lighting('ambient')
         scalars = a.pointdata[0]
         if scalars is None:
-            print("Error in legosurface(): no scalars found!")
+            #print("Error in legosurface(): no scalars found!")
             return a
-        a.cmap(cmap, scalars, vmin=srng[0], vmax=srng[1])
         a.mapPointsToCells()
         return a
 
@@ -2074,7 +2117,7 @@ def streamLines(domain, probe,
     elif integrator == 'rk45':
         st.SetIntegratorTypeToRungeKutta45()
     else:
-        colors.printc("Error in streamlines, unknown integrator", integrator, c='r')
+        vedo.logger.error(f"in streamlines, unknown integrator {integrator}")
 
     st.Update()
     output = st.GetOutput()
@@ -2151,7 +2194,3 @@ def streamLines(domain, probe,
 #         ef.AddCellType(ct)
 #     ef.Update()
 #     return Mesh(ef.GetOutput())
-
-
-
-
