@@ -16,22 +16,31 @@ Command Line Interface module
     alias vs='vedo -i --search '  # to search for a string in examples
     alias ve='vedo --eog '        # to view single and multiple images (press h for help)
 """
+import argparse
+import glob
+import os
+import sys
 
-import sys, argparse, os, glob
 import numpy as np
+import vedo
+import vedo.applications as applications
 import vtk
-
-from vedo import io, load, settings, __version__
-from vedo.plotter import Plotter
-from vedo.utils import printInfo, isSequence, humansort
-from vedo.colors import printc, getColor
+from vedo import __version__
+from vedo import io
+from vedo import load
+from vedo import settings
+from vedo.colors import getColor
+from vedo.colors import printc
+from vedo.docs import tips
 from vedo.mesh import Mesh
+from vedo.picture import Picture
+from vedo.plotter import Plotter
 from vedo.tetmesh import TetMesh
 from vedo.ugrid import UGrid
+from vedo.utils import humansort
+from vedo.utils import isSequence
+from vedo.utils import printInfo
 from vedo.volume import Volume
-from vedo.docs import tips
-from vedo.picture import Picture
-import vedo.applications as applications
 
 
 __all__ = []
@@ -42,8 +51,8 @@ def execute_cli():
     parser = get_parser()
     args = parser.parse_args()
 
-    if "/vedo/vedo" in settings.installdir:
-        settings.installdir = settings.installdir.replace('vedo/','').replace('vedo\\','')
+    if "/vedo/vedo" in vedo.installdir:
+        vedo.installdir = vedo.installdir.replace('vedo/','').replace('vedo\\','')
 
     if args.info is not None:
         exe_info(args)
@@ -101,8 +110,8 @@ def get_parser():
     pr.add_argument("--mode",                       help="volume rendering style (composite/maxproj/...)", default=0, metavar='')
     pr.add_argument("--cmap",                       help="volume rendering color map name", default='jet', metavar='')
     pr.add_argument("-e", "--edit",                 help="free-hand edit the input Mesh", action="store_true")
-    pr.add_argument("--slicer",                     help="slicer Mode for volumetric data", action="store_true")
     pr.add_argument("--slicer2d",                   help="2D Slicer Mode for volumetric data", action="store_true")
+    pr.add_argument("--slicer3d",                   help="3D Slicer Mode for volumetric data", action="store_true")
     pr.add_argument("--lego",                       help="voxel rendering for 3D image files", action="store_true")
     pr.add_argument("-r", "--run",                  help="run example from vedo/examples", metavar='')
     pr.add_argument("--search",           type=str, help="search/grep for word in vedo examples", default='', metavar='')
@@ -130,7 +139,7 @@ def exe_info(args):
             else:
                 printInfo(A)
         except:
-            printc("Could not load:", file, "skip.", c="r")
+            vedo.logger.error(f"Could not load {file}, skip.")
 
     printc("_" * 65, bold=0)
     printc("vedo version      :", __version__, invert=1, end='   ')
@@ -138,7 +147,7 @@ def exe_info(args):
     printc("vtk version       :", vtk.vtkVersion().GetVTKVersion())
     printc("python version    :", sys.version.replace("\n", ""))
     printc("python interpreter:", sys.executable)
-    printc("vedo installation :", settings.installdir)
+    printc("vedo installation :", vedo.installdir)
     try:
         import platform
         printc("system            :", platform.system(),
@@ -169,7 +178,7 @@ def exe_info(args):
 
 #################################################################################################
 def exe_run(args):
-    expath = os.path.join(settings.installdir, "examples", "**", "*.py")
+    expath = os.path.join(vedo.installdir, "examples", "**", "*.py")
     exfiles = [f for f in glob.glob(expath, recursive=True)]
     f2search = os.path.basename(args.run).lower()
     matching = [s for s in exfiles if (f2search in os.path.basename(s).lower() and "__" not in s)]
@@ -177,7 +186,7 @@ def exe_run(args):
     nmat = len(matching)
     if nmat == 0:
         printc("No matching example found containing string:", args.run, c=1)
-        printc(" Current installation directory is:", settings.installdir, c=1)
+        printc(" Current installation directory is:", vedo.installdir, c=1)
         exit(1)
 
     if nmat > 1:
@@ -257,7 +266,7 @@ def exe_convert(args):
 
 ##############################################################################################
 def exe_search(args):
-    expath = os.path.join(settings.installdir, "examples", "**", "*.py")
+    expath = os.path.join(vedo.installdir, "examples", "**", "*.py")
     exfiles = [f for f in sorted(glob.glob(expath, recursive=True))]
     pattern = args.search
     if args.no_camera_share:
@@ -385,6 +394,10 @@ def exe_eog(args):
                     p.mirror()
             elif event.keyPressed=="t":
                     p.rotate(90)
+            elif event.keyPressed=="f":
+                    p.flip()
+            elif event.keyPressed=="i":
+                    p.invert()
             elif event.keyPressed=="k":
                     p.enhance()
             elif event.keyPressed=="s":
@@ -401,8 +414,10 @@ def exe_eog(args):
                 printc('Press:')
                 printc('  up/down     to modify level (or drag mouse)')
                 printc('  left/right  to modify window')
-                printc('  m           to mirror image')
+                printc('  m           to mirror image horizontally')
+                printc('  f           to flip image vertically')
                 printc('  t           to rotate image by 90 deg')
+                printc('  i           to invert colors')
                 printc('  k           to enhance b&w image')
                 printc('  s           to apply gaussian smoothing')
                 printc('  S           to save image as png')
@@ -410,16 +425,23 @@ def exe_eog(args):
 
             plt.render()
 
-    pics = load(files)
-    if isinstance(pics, Picture):
-        pics = [pics]
-    if pics is None:
-        return
+    pics =[]
+    for f in files:
+        if os.path.isfile(f):
+            try:
+                pic = Picture(f)
+                if pic:
+                    pics.append(pic)
+            except:
+                vedo.logger.error(f"Could not load image {f}")
+        else:
+            vedo.logger.error(f"Could not load image {f}")
+
     n = len(pics)
-    pic = pics[0]
-    if pic is None:
-        printc("Could not load image.", c='r')
+    if not n:
         return
+
+    pic = pics[0]
     lev, win = pic.level(), pic.window()
 
     if n > 1:
@@ -496,20 +518,20 @@ def draw_scene(args):
             printc("Warning: option '-n' allows a maximum of 200 files", c=1)
             printc("         you are trying to load ", nfiles, " files.\n", c=1)
             N = 200
-        vp = Plotter(size=wsize, N=N, bg=args.background, bg2=args.background_grad)
+        plt = Plotter(size=wsize, N=N, bg=args.background, bg2=args.background_grad)
         settings.immediateRendering=False
-        vp.axes = args.axes_type
+        plt.axes = args.axes_type
         for i in range(N):
-            vp.addHoverLegend(at=i)
+            plt.addHoverLegend(at=i)
         if args.axes_type == 4 or args.axes_type == 5:
-            vp.axes = 0
+            plt.axes = 0
     else:
         N = nfiles
-        vp = Plotter(size=wsize, bg=args.background, bg2=args.background_grad)
-        vp.axes = args.axes_type
-        vp.addHoverLegend()
+        plt = Plotter(size=wsize, bg=args.background, bg2=args.background_grad)
+        plt.axes = args.axes_type
+        plt.addHoverLegend()
 
-    vp.sharecam = not args.no_camera_share
+    plt.sharecam = not args.no_camera_share
 
     wire = False
     if args.wireframe:
@@ -523,8 +545,7 @@ def draw_scene(args):
         vol = io.load(args.files[0], force=args.reload)
 
         if not isinstance(vol, Volume):
-            printc("Type Error: expected a Volume but loaded", type(vol),
-                   'object.', c=1)
+            vedo.logger.error(f"expected a Volume but loaded a {type(vol)} object")
             return
 
         sp = vol.spacing()
@@ -532,17 +553,17 @@ def draw_scene(args):
         vol.mode(int(args.mode)).color(args.cmap).jittering(True)
         # if args.lighting !='default':
         vol.lighting(args.lighting).jittering(True)
-        vp = applications.RayCastPlotter(vol)
-        vp.show(viewup="z", interactive=True)
-        vp.sliders[0][0].SetEnabled(False)
-        vp.sliders[1][0].SetEnabled(False)
-        vp.sliders[2][0].SetEnabled(False)
+        plt = applications.RayCastPlotter(vol)
+        plt.show(viewup="z", interactive=True)
+        plt.sliders[0][0].SetEnabled(False)
+        plt.sliders[1][0].SetEnabled(False)
+        plt.sliders[2][0].SetEnabled(False)
         return
 
     ##########################################################
-    # special case of SLC/TIFF/DICOM volumes with --slicer option
-    elif args.slicer:
-        # print('DEBUG special case of SLC/TIFF/DICOM volumes with --slicer option')
+    # special case of SLC/TIFF/DICOM volumes with --slicer3d option
+    elif args.slicer3d:
+        # print('DEBUG special case of SLC/TIFF/DICOM volumes with --slicer3d option')
 
         useSlider3D = False
         if args.axes_type == 4:
@@ -556,9 +577,9 @@ def draw_scene(args):
         sp = vol.spacing()
         vol.spacing([sp[0]*args.x_spacing, sp[1]*args.y_spacing, sp[2]*args.z_spacing])
 
-        settings.plotter_instance = None # reset
+        vedo.plotter_instance = None # reset
 
-        plt = applications.SlicerPlotter(
+        plt = applications.Slicer3DPlotter(
                      vol,
                      bg='white', bg2='lb',
                      useSlider3D=useSlider3D,
@@ -568,26 +589,25 @@ def draw_scene(args):
                      clamp=True,
                      size=(1000,800),
         )
-        plt.show()
         return
 
     ########################################################################
     elif args.edit:
         # print('edit mode for meshes and pointclouds')
-        settings.plotter_instance = None # reset
+        vedo.plotter_instance = None # reset
         settings.useParallelProjection = True
 
         try:
             m = Mesh(args.files[0], alpha=args.alpha/2, c=args.color)
         except AttributeError:
-            printc("In edit mode, input file must be a point cloud or polygonal mesh. Exit.", c='r')
+            vedo.logger.critical("In edit mode, input file must be a point cloud or polygonal mesh.")
             return
 
-        vp = applications.FreeHandCutPlotter(m, splined=True)
-        vp.addHoverLegend()
+        plt = applications.FreeHandCutPlotter(m, splined=True)
+        plt.addHoverLegend()
         if not args.background_grad:
             args.background_grad = None
-        vp.start(axes=1, bg=args.background, bg2=args.background_grad)
+        plt.start(axes=1, bg=args.background, bg2=args.background_grad)
 
     ########################################################################
     elif args.slicer2d:
@@ -598,9 +618,10 @@ def draw_scene(args):
         vol.cmap('bone_r')
         sp = vol.spacing()
         vol.spacing([sp[0]*args.x_spacing, sp[1]*args.y_spacing, sp[2]*args.z_spacing])
-        settings.plotter_instance = None # reset
-        plt = applications.Slicer2d(vol)
-        plt.interactor.Start()
+        vedo.plotter_instance = None # reset
+
+        plt = applications.Slicer2DPlotter(vol, axes=7)
+        plt.close()
         return
 
 
@@ -620,12 +641,15 @@ def draw_scene(args):
         vol.spacing([sp[0]*args.x_spacing, sp[1]*args.y_spacing, sp[2]*args.z_spacing])
         if not args.color:
             args.color = 'gold'
-        vp = applications.IsosurfaceBrowser(vol,
+        plt = applications.IsosurfaceBrowser(vol,
                                             lego=args.lego,
                                             c=args.color,
                                             cmap=args.cmap,
-                                            delayed=args.lego)
-        vp.show(zoom=args.zoom, viewup="z")
+                                            delayed=args.lego,
+                                            precompute=True,
+                                            progress=True,
+        )
+        plt.show(zoom=args.zoom, viewup="z")
         return
 
 
@@ -648,7 +672,7 @@ def draw_scene(args):
                 objct.show(mode=interactor_mode)
                 return
             else:                             # loading a set of meshes
-                vp.show(objct, mode=interactor_mode)
+                plt.show(objct, mode=interactor_mode)
                 return
         #########################################################
 
@@ -697,41 +721,42 @@ def draw_scene(args):
             if args.multirenderer_mode:
                 try:
                     ds = actor.diagonalSize() * 3
-                    vp.camera.SetClippingRange(0, ds)
-                    vp.show(actor, at=i, interactive=False, zoom=args.zoom, mode=interactor_mode)
-                    vp.actors = actors
+                    plt.camera.SetClippingRange(0, ds)
+                    plt.show(actor, at=i, interactive=False, zoom=args.zoom, mode=interactor_mode)
+                    plt.actors = actors
                 except AttributeError:
                     # wildcards in quotes make glob return actor as a list :(
-                    printc("Please do not use wildcards within single or double quotes!", c='r')
+                    vedo.logger.error("Please do not use wildcards within single or double quotes")
 
         if args.multirenderer_mode:
-            vp.interactor.Start()
+            plt.interactor.Start()
 
         else:
 
             # scene is empty
             if all(a is None for a in actors):
-                printc("..could not load file(s). Quit.", c='r')
+                vedo.logger.error("Could not load file(s). Quit.")
                 return
 
-            vp.show(actors, interactive=True, zoom=args.zoom, mode=interactor_mode)
+            plt.show(actors, interactive=True, zoom=args.zoom, mode=interactor_mode)
         return
 
     ########################################################################
     # scrolling mode  -s
     else:
         #print("DEBUG simple browser mode  -s")
-        if vp.axes==4:
-            vp.axes=1
+        if plt.axes==4:
+            plt.axes=1
 
-        acts = vp.load(args.files, force=args.reload)
+        acts = plt.load(args.files, force=args.reload)
         for a in acts:
             if hasattr(a, 'c'): #Picture doesnt have it
                 a.c(args.color)
             a.alpha(args.alpha)
 
-        applications.Browser(acts)
-        vp.show(interactive=True, zoom=args.zoom)
+        plt = applications.Browser(acts, axes=1)
+        plt.show(zoom=args.zoom).close()
+
 
 ########################################################################
 def exe_gui(args):
@@ -835,8 +860,8 @@ def exe_gui(args):
                 "composite",
                 "maximum proj",
                 "lego",
-                "slicer",
                 "slicer2d",
+                "slicer3d",
             )
             self.modeCB = Combobox(self, state="readonly", values=modevalues, width=20)
             self.modeCB.current(0)
@@ -964,7 +989,7 @@ def exe_gui(args):
 
             args.ray_cast_mode = False
             args.lego = False
-            args.slicer = False
+            args.slicer3d = False
             args.slicer2d = False
             args.lego = False
             args.mode = 0
@@ -974,8 +999,8 @@ def exe_gui(args):
             elif self.modeCB.get() == "maximum proj":
                 args.ray_cast_mode = True
                 args.mode = 1
-            elif self.modeCB.get() == "slicer":
-                args.slicer = True
+            elif self.modeCB.get() == "slicer3d":
+                args.slicer3d = True
             elif self.modeCB.get() == "slicer2d":
                 args.slicer2d = True
             elif self.modeCB.get() == "lego":
@@ -991,8 +1016,8 @@ def exe_gui(args):
             draw_scene(args)
             if os.name == "nt":
                 exit()
-            if settings.plotter_instance:
-                settings.plotter_instance.close()
+            if vedo.plotter_instance:
+                vedo.plotter_instance.close()
 
     root = Tk()
     root.geometry("360x500")
@@ -1012,6 +1037,3 @@ def exe_gui(args):
         print("Already", len(app.filenames), "files loaded.")
 
     root.mainloop()
-
-
-
