@@ -1,15 +1,12 @@
-import vtk
-import numpy as np
-import sys, os, time
-import vedo.docs as docs
-import vedo.settings as settings
+import os
+import sys
+import time
 
-__doc__ = (
-    """
-Colors definitions and printing methods.
-"""
-    + docs._defs
-)
+import numpy as np
+import vedo
+import vtk
+
+__doc__ = ("Colors definitions and printing methods." )
 
 __all__ = [
     "printc",
@@ -21,7 +18,6 @@ __all__ = [
     "buildLUT",
 ]
 
-_global_start_time = time.time()
 
 try:
     import matplotlib.cm as cm_mpl
@@ -598,8 +594,7 @@ def getColor(rgb=None, hsv=None):
             if c.lower() in color_nicks.keys():
                 c = color_nicks[c.lower()]
             else:
-                print("Unknown color nickname:", c)
-                print("Available abbreviations:", color_nicks)
+                vedo.logger.warning(f"Unknown color nickname {c}\nAvailable abbreviations: {color_nicks}")
                 return (0.5, 0.5, 0.5)
 
         if c.lower() in colors.keys():  # matplotlib name color
@@ -611,7 +606,7 @@ def getColor(rgb=None, hsv=None):
             rgb255 = list(int(h[i : i + 2], 16) for i in (0, 2, 4))
             rgbh = np.array(rgb255) / 255.0
             if np.sum(rgbh) > 3:
-                print("Error in getColor(): Wrong hex color", c)
+                vedo.logger.error(f"in getColor(): Wrong hex color {c}")
                 return (0.5, 0.5, 0.5)
             return tuple(rgbh)
 
@@ -718,10 +713,10 @@ def colorMap(value, name="jet", vmin=None, vmax=None):
         values = (values - vmin) / (vmax - vmin)
     else:
         if vmin is None:
-            printc("In colorMap(): must specify vmin! Assume 0.0", c='r')
+            vedo.logger.warning("in colorMap() you must specify vmin! Assume 0.")
             vmin = 0
         if vmax is None:
-            printc("In colorMap(): must specify vmax! Assume 1.0", c='r')
+            vedo.logger.warning("in colorMap() you must specify vmax! Assume 1.")
             vmax = 1
         values = [(value - vmin) / (vmax - vmin)]
 
@@ -742,8 +737,8 @@ def colorMap(value, name="jet", vmin=None, vmax=None):
         try:
             cmap = cmaps[name]
         except KeyError:
-            printc("In colorMap(): no color map with name:", name, 'or', name+'_r', c='r')
-            printc("Available color maps are:\n", cmaps.keys(), c='y', bold=False)
+            vedo.logger.error(f"in colorMap(), no color map with name {name} or {name}_r")
+            vedo.logger.error(f"Available color maps are:\n{cmaps.keys()}")
             return np.array([0.5,0.5,0.5])
 
         result = []
@@ -917,11 +912,11 @@ def printc(*strings,
 
     |colorprint| |colorprint.py|_
     """
-    if not settings.enablePrintColor:
+    if not vedo.settings.enablePrintColor:
         print(*strings, end=end, flush=flush)
         return
 
-    if not settings.notebookBackend:
+    if not vedo.notebookBackend:
         if not _terminal_has_colors:
             print(*strings, end=end, flush=flush)
             return
@@ -1072,17 +1067,7 @@ def printd(*strings, q=False):
             except:
                 pass
 
-    print("    \x1b[1m\x1b[37mElapsed time:\x1b[0m\x1b[37m",
-          str(time.time()-_global_start_time)[:6], 's'+reset)
     if q:
         print(f"    \x1b[1m\x1b[37mExiting python now (q={bool(q)}).\x1b[0m\x1b[37m")
         exit(0)
     sys.stdout.flush()
-
-
-
-
-
-
-
-
