@@ -1,27 +1,25 @@
-import vtk
 import numpy as np
-
-import vedo.utils as utils
-from vedo.utils import printHistogram, ProgressBar
-
-import vedo.docs as docs
-
-from vedo.colors import printc
-
-import vedo.settings as settings
-from vedo.settings import dataurl, embedWindow
-
-from vedo.mesh import Mesh
-
-from vedo.io import load, screenshot, Video, exportWindow, download
-
+import vedo
 import vedo.shapes as shapes
-from vedo.shapes import Text3D, Text2D, Latex
-
-from vedo.plotter import show, clear, Plotter
+import vedo.utils as utils
+import vtk
+from vedo.colors import printc
+from vedo.io import download
+from vedo.io import exportWindow
+from vedo.io import load
+from vedo.io import screenshot
+from vedo.io import Video
+from vedo.mesh import Mesh
+from vedo.plotter import clear
 from vedo.plotter import interactive
-
+from vedo.plotter import Plotter
+from vedo.plotter import show
 from vedo.pyplot import histogram
+from vedo.shapes import Latex
+from vedo.shapes import Text2D
+from vedo.shapes import Text3D
+from vedo.utils import printHistogram
+from vedo.utils import ProgressBar
 
 # Install fenics with commands (e.g. in Anaconda3):
 #         conda install -c conda-forge fenics
@@ -95,7 +93,7 @@ Image Gallery
 | Customizing axes style and appearance           |The wave equation in arbitrary nr. of dimensions |
 +-------------------------------------------------+-------------------------------------------------+
 """
-    + docs._defs
+    + vedo.docs._defs
 )
 
 __all__ = [
@@ -112,12 +110,10 @@ __all__ = [
     "Text3D",
     "Text2D",
     "Latex",
-    "dataurl",
     "screenshot",
     "Video",
     "exportWindow",
     "interactive",
-    "embedWindow",
 ]
 
 
@@ -230,7 +226,7 @@ def _compute_uvalues(u, mesh):
         coords = mesh.geometry.points
 
     if u_values.shape[0] != coords.shape[0]:
-        printc('Warning: mismatch in vedo.dolfin._compute_uvalues()', c='r')
+        vedo.logger.warning("mismatch in vedo.dolfin._compute_uvalues")
         u_values = np.array([u(p) for p in coords])
     return u_values
 
@@ -426,36 +422,25 @@ def plot(*inputobj, **options):
     exterior = options.pop("exterior", False)
     fast = options.pop("fast", False)
     returnActorsNoShow = options.pop("returnActorsNoShow", False)
+    at = options.pop("at", 0)
 
     # refresh axes titles for axes type = 8 (vtkCubeAxesActor)
-    settings.xtitle = options.pop("xtitle", 'x')
-    settings.ytitle = options.pop("ytitle", 'y')
-    settings.ztitle = options.pop("ztitle", 'z')
-    if settings.plotter_instance:
-        if settings.ytitle!='x':
-            if 'at' in options.keys():
-                at = options['at']
-            else:
-                at = 0
-            aet = settings.plotter_instance.axes_instances
+    xtitle = options.pop("xtitle", 'x')
+    ytitle = options.pop("ytitle", 'y')
+    ztitle = options.pop("ztitle", 'z')
+    if vedo.plotter_instance:
+        if xtitle!='x':
+            aet = vedo.plotter_instance.axes_instances
             if len(aet)>at and isinstance(aet[at], vtk.vtkCubeAxesActor):
-                aet[at].SetXTitle(settings.xtitle)
-        if settings.ytitle!='y':
-            if 'at' in options.keys():
-                at = options['at']
-            else:
-                at = 0
-            aet = settings.plotter_instance.axes_instances
+                aet[at].SetXTitle(xtitle)
+        if ytitle!='y':
+            aet = vedo.plotter_instance.axes_instances
             if len(aet)>at and isinstance(aet[at], vtk.vtkCubeAxesActor):
-                aet[at].SetYTitle(settings.ytitle)
-        if settings.ytitle!='z':
-            if 'at' in options.keys():
-                at = options['at']
-            else:
-                at = 0
-            aet = settings.plotter_instance.axes_instances
+                aet[at].SetYTitle(ytitle)
+        if ztitle!='z':
+            aet = vedo.plotter_instance.axes_instances
             if len(aet)>at and isinstance(aet[at], vtk.vtkCubeAxesActor):
-                aet[at].SetZTitle(settings.ztitle)
+                aet[at].SetZTitle(ztitle)
 
 
     # change some default to emulate standard behaviours
@@ -531,12 +516,12 @@ def plot(*inputobj, **options):
 
     #################################################################
     actors = []
-    if settings.plotter_instance:
+    if vedo.plotter_instance:
         if add:
-            actors = settings.plotter_instance.actors
+            actors = vedo.plotter_instance.actors
         elif at==0: # just remove scalarbars
-            for sb in settings.plotter_instance.scalarbars:
-                settings.plotter_instance.renderer.RemoveActor(sb)
+            for sb in vedo.plotter_instance.scalarbars:
+                vedo.plotter_instance.renderer.RemoveActor(sb)
 
     if mesh and ('mesh' in mode or 'color' in mode or 'displace' in mode):
 
@@ -661,16 +646,16 @@ def plot(*inputobj, **options):
         actors.append(text)
 
     if 'at' in options.keys() and 'interactive' not in options.keys():
-        if settings.plotter_instance:
-            N = settings.plotter_instance.shape[0]*settings.plotter_instance.shape[1]
+        if vedo.plotter_instance:
+            N = vedo.plotter_instance.shape[0]*vedo.plotter_instance.shape[1]
             if options['at'] == N-1:
                 options['interactive'] = True
 
-    # if settings.plotter_instance:
-    #     for a2 in settings.collectable_actors:
+    # if vedo.plotter_instance:
+    #     for a2 in vedo.collectable_actors:
     #         if isinstance(a2, vtk.vtkCornerAnnotation):
     #             if 0 in a2.renderedAt: # remove old message
-    #                 settings.plotter_instance.remove(a2)
+    #                 vedo.plotter_instance.remove(a2)
     #                 break
 
     if len(actors)==0:
@@ -744,9 +729,8 @@ class MeshActor(Mesh):
             coords = self.mesh.geometry.points
 
         if coords.shape != deltas.shape:
-            printc("ERROR: Try to move mesh with wrong solution type shape:",
-                  coords.shape, 'vs', deltas.shape, c='r')
-            printc("Mesh is not moved. Try mode='color' in plot().", c='r')
+            vedo.logger.error(f"Try to move mesh with wrong solution type shape {coords.shape} vs {deltas.shape}")
+            vedo.logger.error("Mesh is not moved. Try mode='color' in plot().")
             return
 
         movedpts = coords + deltas
@@ -827,7 +811,7 @@ def MeshLines(*inputobj, **options):
 
     u_values = _compute_uvalues(u,mesh)
     if not utils.isSequence(u_values[0]):
-        printc("\times Error: cannot show Lines for 1D scalar values!", c='r')
+        vedo.logger.error("cannot show Lines for 1D scalar values")
         raise RuntimeError()
 
     endPoints = startPoints + u_values
@@ -868,7 +852,7 @@ def MeshArrows(*inputobj, **options):
 
     u_values = _compute_uvalues(u,mesh)
     if not utils.isSequence(u_values[0]):
-        printc("\times Error: cannot show Arrows for 1D scalar values!", c='r')
+        vedo.logger.error("cannot show Arrows for 1D scalar values")
         raise RuntimeError()
 
     endPoints = startPoints + u_values *scale
@@ -911,7 +895,7 @@ def MeshStreamLines(*inputobj, **options):
 
     u_values = _compute_uvalues(u, mesh)
     if not utils.isSequence(u_values[0]):
-        printc("\times Error: cannot show Arrows for 1D scalar values!", c='r')
+        vedo.logger.error("cannot show Arrows for 1D scalar values")
         raise RuntimeError()
     if u_values.shape[1] == 2:  # u_values is 2D
         u_values = np.insert(u_values, 2, 0, axis=1)  # make it 3d
@@ -947,7 +931,3 @@ def MeshStreamLines(*inputobj, **options):
         str_lns.lw(lw)
 
     return str_lns
-
-
-
-
