@@ -1,19 +1,19 @@
-import vtk
-import os
 import glob
-import numpy as np
+import os
 import time
 
+import numpy as np
 import vedo
-import vedo.utils as utils
 import vedo.colors as colors
+import vedo.docs as docs
+import vedo.utils as utils
+import vtk
+from vedo import settings
 from vedo.assembly import Assembly
 from vedo.mesh import Mesh
-from vedo.pointcloud import Points
 from vedo.picture import Picture
+from vedo.pointcloud import Points
 from vedo.volume import Volume
-import vedo.docs as docs
-import vedo.settings as settings
 
 __doc__ = (
     """
@@ -136,17 +136,19 @@ def load(inputobj, unpack=True, force=False):
                     a = _load_file(fod+'/'+ifile, unpack)
                     acts.append(a)
         else:
-            colors.printc("Error in load(): cannot find", fod, c='r')
+            vedo.logger.error(f"in load(), cannot find {fod}")
 
     if len(acts) == 1:
         if "numpy" in str(type(acts[0])):
             return acts[0]
         if not acts[0]:
-            colors.printc("Error in load(): cannot load", inputobj, c='r')
+            vedo.logger.error(f"in load(), cannot load {inputobj}")
         return acts[0]
+
     elif len(acts) == 0:
-        colors.printc("Error in load(): cannot load", inputobj, c='r')
+        vedo.logger.error(f"in load(), cannot load {inputobj}")
         return None
+
     else:
         return acts
 
@@ -183,7 +185,7 @@ def _load_file(filename, unpack):
         actor = Assembly(wacts)
 
         ################################################################# volumetric:
-    elif fl.endswith(".tif") or fl.endswith(".slc") or fl.endswith(".vti") \
+    elif fl.endswith(".tif") or fl.endswith(".tiff") or fl.endswith(".slc") or fl.endswith(".vti") \
         or fl.endswith(".mhd") or fl.endswith(".nrrd") or fl.endswith(".nii") \
         or fl.endswith(".dem"):
         img = loadImageData(filename)
@@ -237,7 +239,7 @@ def _load_file(filename, unpack):
 
         ################################################################# numpy:
     elif fl.endswith(".npy") or fl.endswith(".npz"):
-        acts = loadNumpy(filename)
+        acts = loadnumpy(filename)
 
         if unpack is False:
             return Assembly(acts)
@@ -304,7 +306,7 @@ def _load_file(filename, unpack):
         routput = reader.GetOutput()
 
         if not routput:
-            colors.printc("\noentry Unable to load", filename, c='r')
+            vedo.logger.error(f"unable to load {filename}")
             return None
 
         if isinstance(routput, vtk.vtkUnstructuredGrid):
@@ -324,7 +326,7 @@ def download(url, force=False, verbose=True):
     """Retrieve a file from a url, save it locally and return its path."""
 
     if not url.startswith('https://'):
-        colors.printc('Invalid URL (must start with https):\n', url, c='r')
+        vedo.logger.error(f"Invalid URL (must start with https):\n{url}")
         return url
     url = url.replace('www.dropbox', 'dl.dropbox')
 
@@ -343,7 +345,7 @@ def download(url, force=False, verbose=True):
 
     if force==False and os.path.exists(tmp_file.name):
         if verbose:
-            colors.printc("using cached file:", tmp_file.name)
+            colors.printc("reusing cached file:", tmp_file.name)
             #colors.printc("     (use force=True to force a new download)")
         return tmp_file.name
 
@@ -381,9 +383,8 @@ def gunzip(filename):
     tmp_file.name = os.path.join(os.path.dirname(tmp_file.name),
                                  os.path.basename(filename).replace('.gz',''))
     inF = gzip.open(filename, "rb")
-    outF = open(tmp_file.name, "wb")
-    outF.write(inF.read())
-    outF.close()
+    with open(tmp_file.name, "wb") as outF:
+        outF.write(inF.read())
     inF.close()
     return tmp_file.name
 
@@ -476,9 +477,8 @@ def load3DS(filename):
 
 def loadOFF(filename):
     """Read the OFF file format."""
-    f = open(filename, "r")
-    lines = f.readlines()
-    f.close()
+    with open(filename, "r") as f:
+        lines = f.readlines()
 
     vertices = []
     faces = []
@@ -560,9 +560,8 @@ def _loadDolfin_old(filename, exterior='dummy'):
         import gzip
 
         inF = gzip.open(filename, "rb")
-        outF = open("/tmp/filename.xml", "wb")
-        outF.write(inF.read())
-        outF.close()
+        with open("/tmp/filename.xml", "wb") as outF:
+            outF.write(inF.read())
         inF.close()
         tree = et.parse("/tmp/filename.xml")
     else:
@@ -645,9 +644,8 @@ def loadPDB(filename, bondScale=1, hydrogenBondScale=1, coilWidth=0.3, helixWidt
 
 def loadNeutral(filename):
     """Reads a `Neutral` tetrahedral file format. Return an ``Mesh`` object."""
-    f = open(filename, "r")
-    lines = f.readlines()
-    f.close()
+    with open(filename, "r") as f:
+        lines = f.readlines()
 
     ncoords = int(lines[0])
     coords = []
@@ -673,9 +671,8 @@ def loadNeutral(filename):
 
 def loadGmesh(filename):
     """Reads a `gmesh` file format. Return an ``Mesh`` object."""
-    f = open(filename, "r")
-    lines = f.readlines()
-    f.close()
+    with open(filename, "r") as f:
+        lines = f.readlines()
 
     nnodes = 0
     index_nodes = 0
@@ -708,9 +705,9 @@ def loadGmesh(filename):
 def loadPCD(filename):
     """Return a ``Mesh`` made of only vertex points
     from `Point Cloud` file format. Return an ``Points`` object."""
-    f = open(filename, "r")
-    lines = f.readlines()
-    f.close()
+    with open(filename, "r") as f:
+        lines = f.readlines()
+
     start = False
     pts = []
     N, expN = 0, 0
@@ -726,12 +723,12 @@ def loadPCD(filename):
         if not start and "DATA ascii" in text:
             start = True
     if expN != N:
-        colors.printc("Mismatch in pcd file", expN, len(pts), c="red")
+        vedo.logger.warning(f"Mismatch in PCD file {expN} != {len(pts)}")
     poly = utils.buildPolyData(pts)
     return Points(poly).pointSize(4)
 
 
-def toNumpy(obj):
+def tonumpy(obj):
     '''Dump a vedo object to numpy format.'''
 
     adict = dict()
@@ -778,12 +775,19 @@ def toNumpy(obj):
 
         adict['pointdata'] = []
         for iname in obj.pointdata.keys():
-            if 'Normals' in iname.lower(): continue
+            if not iname:
+                continue
+            if 'Normals' in iname.lower():
+                continue
             arr = poly.GetPointData().GetArray(iname)
             adict['pointdata'].append([utils.vtk2numpy(arr), iname])
+
         adict['celldata'] = []
         for iname in obj.celldata.keys():
-            if 'Normals' in iname.lower(): continue
+            if not iname:
+                continue
+            if 'Normals' in iname.lower():
+                continue
             arr = poly.GetCellData().GetArray(iname)
             adict['celldata'].append([utils.vtk2numpy(arr), iname])
 
@@ -814,14 +818,14 @@ def toNumpy(obj):
         adict['linewidth'] = None
         if prp.GetEdgeVisibility():
             adict['linewidth'] = obj.lineWidth()
-            adict['linecolor'] = obj.lineColor()
+            adict['linecolor'] = prp.GetEdgeColor()
 
         adict['ambient'] = prp.GetAmbient()
         adict['diffuse'] = prp.GetDiffuse()
         adict['specular'] = prp.GetSpecular()
         adict['specularpower'] = prp.GetSpecularPower()
         adict['specularcolor'] = prp.GetSpecularColor()
-        adict['shading'] = prp.GetInterpolation()
+        adict['shading'] = prp.GetInterpolation() # flat phong..:
         adict['color'] = prp.GetColor()
         adict['lightingIsOn'] = prp.GetLighting()
         adict['backColor'] = None
@@ -829,17 +833,11 @@ def toNumpy(obj):
             adict['backColor'] = obj.GetBackfaceProperty().GetColor()
 
         adict['scalarvisibility'] = obj.mapper().GetScalarVisibility()
-
         adict['texture'] = None
 
-    ######################################################## Mesh
-    if isinstance(obj, Points):
-        adict['type'] = 'Mesh'
-        _fillcommon(obj, adict)
-        _fillmesh(obj, adict)
 
     ######################################################## Assembly
-    elif isinstance(obj, Assembly):
+    if isinstance(obj, Assembly):
         pass
         # adict['type'] = 'Assembly'
         # _fillcommon(obj, adict)
@@ -850,6 +848,12 @@ def toNumpy(obj):
         #         _fillmesh(a, assdict)
         #         adict['actors'].append(assdict)
 
+    ######################################################## Points/Mesh
+    elif isinstance(obj, Points):
+        adict['type'] = 'Mesh'
+        _fillcommon(obj, adict)
+        _fillmesh(obj, adict)
+
     ######################################################## Volume
     elif isinstance(obj, Volume):
         adict['type'] = 'Volume'
@@ -880,9 +884,7 @@ def toNumpy(obj):
     elif isinstance(obj, Picture):
         adict['type'] = 'Picture'
         _fillcommon(obj, adict)
-        adict['array'] = utils.vtk2numpy(obj.inputdata().GetPointData().GetScalars())
-        adict['shape'] = obj.inputdata().GetDimensions()
-        print('toNumpy(): vedo.Picture', obj.shape, obj.GetPosition())
+        adict['array'] = obj.tonumpy()
 
     ######################################################## Text2D
     elif isinstance(obj, vedo.Text2D):
@@ -891,21 +893,21 @@ def toNumpy(obj):
         adict['text'] = obj.text()
         adict['position'] = obj.GetPosition()
         adict['color'] = obj.property.GetColor()
-        adict['font'] =  obj.font()
+        adict['font']  = obj.fontname
         adict['size']  = obj.property.GetFontSize()/22.5
         adict['bgcol'] = obj.property.GetBackgroundColor()
         adict['alpha'] = obj.property.GetBackgroundOpacity()
         adict['frame'] = obj.property.GetFrame()
-        # print('toNumpy(): vedo.Text2D', obj.text()[:10], obj.font(), obj.GetPosition())
+        # print('tonumpy(): vedo.Text2D', obj.text()[:10], obj.font(), obj.GetPosition())
 
     else:
         pass
-        #colors.printc('Unknown object type in toNumpy()', [obj], c='r')
+        #colors.printc('Unknown object type in tonumpy()', [obj], c='r')
 
     return adict
 
 
-def loadNumpy(inobj):
+def loadnumpy(inobj):
     """Load a vedo format file or scene."""
 
     # make sure the numpy file is not containing a scene
@@ -997,6 +999,7 @@ def loadNumpy(inobj):
         if 'pointdata' in keys:
             for psc, pscname in d['pointdata']:
                 msh.addPointArray(psc, pscname)
+
         msh.mapper().ScalarVisibilityOff()     # deactivate scalars
 
         if 'LUT' in keys and 'activedata' in keys and d['activedata']:
@@ -1035,7 +1038,7 @@ def loadNumpy(inobj):
 
     objs = []
     for d in data:
-        #print('loadNumpy:', d)
+        #print('loadnumpy:', d['type'], d)
 
         ### Mesh
         if 'mesh' == d['type'].lower():
@@ -1065,18 +1068,7 @@ def loadNumpy(inobj):
 
         ### Picture
         elif 'picture' == d['type'].lower():
-            shp = d['shape'][1], d['shape'][0]
-            arr0 = d['array']
-            rcv = arr0[:,0].reshape(shp)
-            rcv = np.flip(rcv, 0)
-            gcv = arr0[:,1].reshape(shp)
-            gcv = np.flip(gcv, 0)
-            bcv = arr0[:,2].reshape(shp)
-            bcv = np.flip(bcv, 0)
-            arr = np.array([rcv, gcv, bcv])
-            arr = np.swapaxes(arr, 0, 2)
-            arr = np.swapaxes(arr, 0, 1)
-            vimg = Picture(arr)
+            vimg = Picture(d['array'])
             _loadcommon(vimg, d)
             objs.append(vimg)
 
@@ -1120,7 +1112,7 @@ def loadImageData(filename):
     elif ".slc" in filename.lower():
         reader = vtk.vtkSLCReader()
         if not reader.CanReadFile(filename):
-            colors.printc("\prohibited Sorry bad slc file " + filename, c='r')
+            vedo.logger.error(f"sorry, bad SLC file {filename}")
             return None
     elif ".vti" in filename.lower():
         reader = vtk.vtkXMLImageDataReader()
@@ -1133,7 +1125,7 @@ def loadImageData(filename):
     elif ".nrrd" in filename.lower():
         reader = vtk.vtkNrrdReader()
         if not reader.CanReadFile(filename):
-            colors.printc("\prohibited Sorry bad nrrd file " + filename, c='r')
+            vedo.logger.error(f"sorry, bad NRRD file {filename}")
             return None
     reader.SetFileName(filename)
     reader.Update()
@@ -1224,89 +1216,87 @@ def write(objct, fileoutput, binary=True):
             objslist = [objct]
         dicts2save = []
         for obj in objslist:
-            dicts2save.append( toNumpy(obj) )
+            dicts2save.append( tonumpy(obj) )
         np.save(fileoutput, dicts2save)
         return dicts2save
 
     elif fr.endswith(".obj"):
-        outF = open(fileoutput, "w")
-        outF.write('# OBJ file format with ext .obj\n')
-        outF.write('# File generated by vedo\n')
+        with open(fileoutput, "w") as outF:
+            outF.write('# OBJ file format with ext .obj\n')
+            outF.write('# File generated by vedo\n')
 
-        for p in objct.points():
-            outF.write("v {:.5g} {:.5g} {:.5g}\n".format(*p))
+            for p in objct.points():
+                outF.write("v {:.5g} {:.5g} {:.5g}\n".format(*p))
 
-        # pdata = objct.polydata().GetPointData().GetScalars()
-        # if pdata:
-        #     ndata = vtk_to_numpy(pdata)
-        #     for vd in ndata:
-        #         outF.write('vp '+ str(vd) +'\n')
+            # pdata = objct.polydata().GetPointData().GetScalars()
+            # if pdata:
+            #     ndata = vtk_to_numpy(pdata)
+            #     for vd in ndata:
+            #         outF.write('vp '+ str(vd) +'\n')
 
-        #ptxt = objct.polydata().GetPointData().GetTCoords() # not working
-        #if ptxt:
-        #    ntxt = vtk_to_numpy(ptxt)
-        #    print(len(objct.faces()), objct.points().shape, ntxt.shape)
-        #    for vt in ntxt:
-        #        outF.write('vt '+ str(vt[0]) +" "+ str(vt[1])+ ' 0\n')
+            #ptxt = objct.polydata().GetPointData().GetTCoords() # not working
+            #if ptxt:
+            #    ntxt = vtk_to_numpy(ptxt)
+            #    print(len(objct.faces()), objct.points().shape, ntxt.shape)
+            #    for vt in ntxt:
+            #        outF.write('vt '+ str(vt[0]) +" "+ str(vt[1])+ ' 0\n')
 
-        for i,f in enumerate(objct.faces()):
-            fs = ''
-            for fi in f:
-                fs += " {:d}".format(fi+1)
-            outF.write('f' + fs + '\n')
+            for i,f in enumerate(objct.faces()):
+                fs = ''
+                for fi in f:
+                    fs += " {:d}".format(fi+1)
+                outF.write('f' + fs + '\n')
 
-        for l in objct.lines():
-            ls = ''
-            for li in l:
-                ls += str(li+1)+" "
-            outF.write('l '+ ls + '\n')
+            for l in objct.lines():
+                ls = ''
+                for li in l:
+                    ls += str(li+1)+" "
+                outF.write('l '+ ls + '\n')
 
-        outF.close()
         return objct
 
     elif fr.endswith(".xml"):  # write tetrahedral dolfin xml
         vertices = objct.points().astype(str)
         faces = np.array(objct.faces()).astype(str)
         ncoords = vertices.shape[0]
-        outF = open(fileoutput, "w")
-        outF.write('<?xml version="1.0" encoding="UTF-8"?>\n')
-        outF.write('<dolfin xmlns:dolfin="http://www.fenicsproject.org">\n')
+        with open(fileoutput, "w") as outF:
+            outF.write('<?xml version="1.0" encoding="UTF-8"?>\n')
+            outF.write('<dolfin xmlns:dolfin="http://www.fenicsproject.org">\n')
 
-        if len(faces[0]) == 4:# write tetrahedral mesh
-            ntets = faces.shape[0]
-            outF.write('  <mesh celltype="tetrahedron" dim="3">\n')
-            outF.write('    <vertices size="' + str(ncoords) + '">\n')
-            for i in range(ncoords):
-                x, y, z = vertices[i]
-                outF.write('      <vertex index="'+str(i)+'" x="'+x+'" y="'+y+'" z="'+z+'"/>\n')
-            outF.write('    </vertices>\n')
-            outF.write('    <cells size="' + str(ntets) + '">\n')
-            for i in range(ntets):
-                v0, v1, v2, v3 = faces[i]
-                outF.write('     <tetrahedron index="'+str(i)
-                           + '" v0="'+v0+'" v1="'+v1+'" v2="'+v2+'" v3="'+v3+'"/>\n')
+            if len(faces[0]) == 4:# write tetrahedral mesh
+                ntets = faces.shape[0]
+                outF.write('  <mesh celltype="tetrahedron" dim="3">\n')
+                outF.write('    <vertices size="' + str(ncoords) + '">\n')
+                for i in range(ncoords):
+                    x, y, z = vertices[i]
+                    outF.write('      <vertex index="'+str(i)+'" x="'+x+'" y="'+y+'" z="'+z+'"/>\n')
+                outF.write('    </vertices>\n')
+                outF.write('    <cells size="' + str(ntets) + '">\n')
+                for i in range(ntets):
+                    v0, v1, v2, v3 = faces[i]
+                    outF.write('     <tetrahedron index="'+str(i)
+                               + '" v0="'+v0+'" v1="'+v1+'" v2="'+v2+'" v3="'+v3+'"/>\n')
 
-        elif len(faces[0]) == 3:# write triangle mesh
-            ntri = faces.shape[0]
-            outF.write('  <mesh celltype="triangle" dim="2">\n')
-            outF.write('    <vertices size="' + str(ncoords) + '">\n')
-            for i in range(ncoords):
-                x, y, dummy_z = vertices[i]
-                outF.write('      <vertex index="'+str(i)+'" x="'+x+'" y="'+y+'"/>\n')
-            outF.write('    </vertices>\n')
-            outF.write('    <cells size="' + str(ntri) + '">\n')
-            for i in range(ntri):
-                v0, v1, v2 = faces[i]
-                outF.write('     <triangle index="'+str(i)+'" v0="'+v0+'" v1="'+v1+'" v2="'+v2+'"/>\n')
+            elif len(faces[0]) == 3:# write triangle mesh
+                ntri = faces.shape[0]
+                outF.write('  <mesh celltype="triangle" dim="2">\n')
+                outF.write('    <vertices size="' + str(ncoords) + '">\n')
+                for i in range(ncoords):
+                    x, y, dummy_z = vertices[i]
+                    outF.write('      <vertex index="'+str(i)+'" x="'+x+'" y="'+y+'"/>\n')
+                outF.write('    </vertices>\n')
+                outF.write('    <cells size="' + str(ntri) + '">\n')
+                for i in range(ntri):
+                    v0, v1, v2 = faces[i]
+                    outF.write('     <triangle index="'+str(i)+'" v0="'+v0+'" v1="'+v1+'" v2="'+v2+'"/>\n')
 
-        outF.write('    </cells>\n')
-        outF.write("  </mesh>\n")
-        outF.write("</dolfin>\n")
-        outF.close()
+            outF.write('    </cells>\n')
+            outF.write("  </mesh>\n")
+            outF.write("</dolfin>\n")
         return objct
 
     else:
-        colors.printc("\noentry Unknown format", fileoutput, "file not saved.", c="r")
+        vedo.logger.error(f"Unknown format {fileoutput}, file not saved")
         return objct
 
     try:
@@ -1318,8 +1308,8 @@ def write(objct, fileoutput, binary=True):
         writer.SetInputData(obj)
         writer.SetFileName(fileoutput)
         writer.Write()
-    except Exception as e:
-        colors.printc("\noentry Error saving: " + fileoutput, "\n", e, c="r")
+    except:
+        vedo.logger.error(f"could not save {fileoutput}")
     return objct
 
 
@@ -1345,8 +1335,8 @@ def writeTransform(inobj, filename='transform.mat', comment=''):
     elif isinstance(inobj, vtk.vtkMatrix4x4):
         M = inobj
     else:
-        colors.printc("Error in io.writeTransform: cannot understand input type",
-                      type(inobj), c='r')
+        vedo.logger.error(f"in writeTransform(), cannot understand input type {type(inobj)}")
+
     with open(filename,'w') as f:
         if comment:
             f.write('# '+comment+'\n')
@@ -1410,7 +1400,7 @@ def exportWindow(fileoutput, binary=False):
     ####################################################################
     if fr.endswith(".npy") or fr.endswith(".npz"):
         sdict = dict()
-        plt = settings.plotter_instance
+        plt = vedo.plotter_instance
         sdict['shape'] = plt.shape #todo
         sdict['sharecam'] = plt.sharecam #todo
         sdict['camera'] = dict( pos=plt.camera.GetPosition(),
@@ -1422,9 +1412,6 @@ def exportWindow(fileoutput, binary=False):
         sdict['size'] = plt.size
         sdict['axes'] = plt.axes
         sdict['title'] = plt.title
-        sdict['xtitle'] = plt.xtitle
-        sdict['ytitle'] = plt.ytitle
-        sdict['ztitle'] = plt.ztitle
         sdict['backgrcol'] = colors.getColor(plt.backgrcol)
         sdict['backgrcol2'] = None
         if plt.renderer.GetGradientBackground():
@@ -1433,7 +1420,7 @@ def exportWindow(fileoutput, binary=False):
         sdict['renderLinesAsTubes'] = settings.renderLinesAsTubes
         sdict['hiddenLineRemoval'] = settings.hiddenLineRemoval
         sdict['visibleGridEdges'] = settings.visibleGridEdges
-        sdict['interactorStyle'] = settings.interactorStyle
+        sdict['interactorStyle'] = vedo.interactorStyle
         sdict['useParallelProjection'] = settings.useParallelProjection
         sdict['defaultFont'] = settings.defaultFont
         sdict['objects'] = []
@@ -1445,11 +1432,12 @@ def exportWindow(fileoutput, binary=False):
             a = acts2d.GetNextItem()
             if isinstance(a, vedo.Text2D):
                 allobjs.append(a)
+        allobjs += plt.actors
 
         allobjs = list(set(allobjs)) # make sure its unique
 
         for a in allobjs:
-            sdict['objects'].append(toNumpy(a))
+            sdict['objects'].append(tonumpy(a))
 
         if fr.endswith(".npz"):
             np.savez_compressed(fileoutput, vedo_scenes=[sdict])
@@ -1460,27 +1448,27 @@ def exportWindow(fileoutput, binary=False):
     elif fr.endswith(".x3d"):
         from vedo.docs import x3d_html
 
-        obj = list(set(settings.plotter_instance.getMeshes()+
-                       settings.plotter_instance.actors))
-        if len(settings.plotter_instance.axes_instances):
-            obj.append(settings.plotter_instance.axes_instances[0])
+        obj = list(set(vedo.plotter_instance.getMeshes()+
+                       vedo.plotter_instance.actors))
+        if len(vedo.plotter_instance.axes_instances):
+            obj.append(vedo.plotter_instance.axes_instances[0])
         for a in obj:
             if isinstance(a, Mesh):
                 newa = a.clone(transformed=True)
-                settings.plotter_instance.remove(a, render=False).add(newa, render=False)
+                vedo.plotter_instance.remove(a, render=False).add(newa, render=False)
 
             elif isinstance(a, Assembly):
                 for b in a.unpack():
                     if b:
                         newb = b.clone(transformed=True)
-                        settings.plotter_instance.add(newb, render=False)
-                settings.plotter_instance.remove(a)
-        settings.plotter_instance.render()
+                        vedo.plotter_instance.add(newb, render=False)
+                vedo.plotter_instance.remove(a)
+        vedo.plotter_instance.render()
 
         exporter = vtk.vtkX3DExporter()
         exporter.SetBinary(binary)
         exporter.FastestOff()
-        exporter.SetInput(settings.plotter_instance.window)
+        exporter.SetInput(vedo.plotter_instance.window)
         exporter.SetFileName(fileoutput)
 #        exporter.WriteToOutputStringOn() # see below
         exporter.Update()
@@ -1510,32 +1498,29 @@ def exportWindow(fileoutput, binary=False):
 #            f.write(l)
 
         x3d_html = x3d_html.replace("~fileoutput", fileoutput)
-        wsize = settings.plotter_instance.window.GetSize()
+        wsize = vedo.plotter_instance.window.GetSize()
         x3d_html = x3d_html.replace("~width", str(wsize[0]))
         x3d_html = x3d_html.replace("~height", str(wsize[1]))
-        outF = open(fileoutput.replace('.x3d', '.html'), "w")
-        outF.write(x3d_html)
-        outF.close()
-        colors.printc("\save Saved files:", fileoutput,
-                      fileoutput.replace('.x3d', '.html'), c="g")
+        with open(fileoutput.replace('.x3d', '.html'), "w") as outF:
+            outF.write(x3d_html)
+            vedo.logger.info(f"Saved files {fileoutput} and {fileoutput.replace('.x3d','.html')}")
 
     ####################################################################
     elif fr.endswith(".html"):
         from vedo.backends import getNotebookBackend
 
-        savebk = settings.notebookBackend
-        settings.notebookBackend='k3d'
-        plt = getNotebookBackend(settings.plotter_instance.actors, 1.5, '')
+        savebk = vedo.notebookBackend
+        vedo.notebookBackend='k3d'
+        plt = getNotebookBackend(vedo.plotter_instance.actors, 1.5, '')
 
         with open(fileoutput,'w') as fp:
             fp.write(plt.get_snapshot())
 
-        settings.notebookBackend = savebk
+        vedo.notebookBackend = savebk
 
     else:
-        colors.printc("Export extension", fr.split('.')[-1],
-                      "is not supported.", c='r')
-    return settings.plotter_instance
+        vedo.logger.error(f"export extension {fr.split('.')[-1]} is not supported")
+    return vedo.plotter_instance
 
 
 def importWindow(fileinput, mtlFile=None, texturePath=None):
@@ -1563,7 +1548,7 @@ def importWindow(fileinput, mtlFile=None, texturePath=None):
         if 'visibleGridEdges' in data.keys():
             settings.visibleGridEdges = data['visibleGridEdges']
         if 'interactorStyle' in data.keys():
-            settings.interactorStyle = data['interactorStyle']
+            vedo.interactorStyle = data['interactorStyle']
         if 'useParallelProjection' in data.keys():
             settings.useParallelProjection = data['useParallelProjection']
         if 'usePolygonOffset' in data.keys():
@@ -1576,6 +1561,8 @@ def importWindow(fileinput, mtlFile=None, texturePath=None):
             settings.interpolateScalarsBeforeMapping = data['interpolateScalarsBeforeMapping']
         if 'defaultFont' in data.keys():
             settings.defaultFont = data['defaultFont']
+        if 'useDepthPeeling' in data.keys():
+            settings.useDepthPeeling = data['useDepthPeeling']
 
         axes = data.pop('axes', 4)
         title = data.pop('title', '')
@@ -1592,9 +1579,6 @@ def importWindow(fileinput, mtlFile=None, texturePath=None):
                      bg=backgrcol,
                      bg2=backgrcol2,
         )
-        plt.xtitle = data.pop('xtitle', 'x')
-        plt.ytitle = data.pop('ytitle', 'y')
-        plt.ztitle = data.pop('ztitle', 'z')
 
         if cam:
             if 'pos' in cam.keys(): plt.camera.SetPosition( cam['pos'] )
@@ -1605,13 +1589,13 @@ def importWindow(fileinput, mtlFile=None, texturePath=None):
             plt.resetcam = False
 
         if 'objects' in data.keys():
-            objs = loadNumpy(data['objects'])
+            objs = loadnumpy(data['objects'])
             if not utils.isSequence(objs):
                objs = [objs]
         else:
             #colors.printc("Trying to import a single mesh.. use load() instead.", c='r')
             #colors.printc(" -> try to load a single object with load().", c='r')
-            objs = [loadNumpy(fileinput)]
+            objs = [loadnumpy(fileinput)]
 
         plt.actors = objs
         return plt
@@ -1647,58 +1631,58 @@ def importWindow(fileinput, mtlFile=None, texturePath=None):
 
 
 ##########################################################
-def screenshot(filename="screenshot.png", scale=None, returnNumpy=False):
+def screenshot(filename="screenshot.png", scale=None, asarray=False):
     """
     Save a screenshot of the current rendering window.
 
     :param int scale: set image magnification
-    :param bool returnNumpy: return a numpy array of the image
+    :param bool asarray: return a numpy array of the image
     """
-    if not settings.plotter_instance or not settings.plotter_instance.window:
-        colors.printc('\bomb screenshot(): Rendering window is not present, skip.', c='r')
-        return settings.plotter_instance
+    if not vedo.plotter_instance or not vedo.plotter_instance.window:
+        vedo.logger.error("in screenshot(), rendering window is not present, skip.")
+        return vedo.plotter_instance
 
     if filename.endswith('.pdf'):
         writer = vtk.vtkGL2PSExporter()
-        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.SetRenderWindow(vedo.plotter_instance.window)
         writer.Write3DPropsAsRasterImageOff()
         writer.SilentOn()
         writer.SetSortToBSP()
         writer.SetFileFormatToPDF()
         writer.SetFilePrefix(filename.replace('.pdf',''))
         writer.Write()
-        return settings.plotter_instance ##########
+        return vedo.plotter_instance ##########
     elif filename.endswith('.svg'):
         writer = vtk.vtkGL2PSExporter()
-        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.SetRenderWindow(vedo.plotter_instance.window)
         writer.Write3DPropsAsRasterImageOff()
         writer.SilentOn()
         writer.SetSortToBSP()
         writer.SetFileFormatToSVG()
         writer.SetFilePrefix(filename.replace('.svg',''))
         writer.Write()
-        return settings.plotter_instance ##########
+        return vedo.plotter_instance ##########
     elif filename.endswith('.eps'):
         writer = vtk.vtkGL2PSExporter()
-        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.SetRenderWindow(vedo.plotter_instance.window)
         writer.Write3DPropsAsRasterImageOff()
         writer.SilentOn()
         writer.SetSortToBSP()
         writer.SetFileFormatToEPS()
         writer.SetFilePrefix(filename.replace('.eps',''))
         writer.Write()
-        return settings.plotter_instance ##########
+        return vedo.plotter_instance ##########
 
     if scale is None:
         scale = settings.screeshotScale
 
     if settings.screeshotLargeImage:
        w2if = vtk.vtkRenderLargeImage()
-       w2if.SetInput(settings.plotter_instance.renderer)
+       w2if.SetInput(vedo.plotter_instance.renderer)
        w2if.SetMagnification(scale)
     else:
         w2if = vtk.vtkWindowToImageFilter()
-        w2if.SetInput(settings.plotter_instance.window)
+        w2if.SetInput(vedo.plotter_instance.window)
         if hasattr(w2if, 'SetScale'):
             w2if.SetScale(scale, scale)
         if settings.screenshotTransparentBackground:
@@ -1706,7 +1690,7 @@ def screenshot(filename="screenshot.png", scale=None, returnNumpy=False):
         w2if.ReadFrontBufferOff()  # read from the back buffer
     w2if.Update()
 
-    if returnNumpy:
+    if asarray:
         w2ifout = w2if.GetOutput()
         npdata = utils.vtk2numpy(w2ifout.GetPointData().GetArray("ImageScalars"))
         npdata = npdata[:,[0,1,2]]
@@ -1730,7 +1714,7 @@ def screenshot(filename="screenshot.png", scale=None, returnNumpy=False):
         writer.SetFileName(filename+'.png')
         writer.SetInputData(w2if.GetOutput())
         writer.Write()
-    return settings.plotter_instance
+    return vedo.plotter_instance
 
 def ask(*question, **kwarg):
     """
@@ -1807,7 +1791,7 @@ class Video:
         self.frames = []
         self.tmp_dir = TemporaryDirectory()
         self.get_filename = lambda x: os.path.join(self.tmp_dir.name, x)
-        colors.printc("\video Video", self.name, "is open...", c="m")
+        colors.printc("\video Video", self.name, "is open... ", c="m", end='')
 
     def addFrame(self):
         """Add frame to current video."""
@@ -1827,24 +1811,24 @@ class Video:
         return self
 
 
-    def action(self, elevation_range=(0,80),
-               azimuth_range=(0,359),
-               zoom=None,
-               cam1=None, cam2=None,
+    def action(self,
+               elevation=(0,80),
+               azimuth=(0,359),
+               cameras=(),
                resetcam=False,
-               ):
+        ):
         """
         Automatic shooting of a static scene by specifying rotation and elevation ranges.
 
-        :param list elevation_range: initial and final elevation angles
+        :param list elevation: initial and final elevation angles
         :param list azimuth_range: initial and final azimuth angles
-        :param float zoom: initial zooming
-        :param cam1 cam2: initial and final camera position, can be dictionary or a vtkCamera
+        :param list cameras: list of cameras to go through, each camera can be dictionary or a vtkCamera
         """
         if not self.duration:
             self.duration = 5
 
-        def buildcam(cm):
+        def build_vtk_cam(cm_input):
+            cm = dict(cm_input)
             cm_pos = cm.pop("pos", None)
             cm_focalPoint = cm.pop("focalPoint", None)
             cm_viewup = cm.pop("viewup", None)
@@ -1864,37 +1848,36 @@ class Video:
             if cm_viewAngle is not None: cm.SetViewAngle(cm_viewAngle)
             return cm
 
-        plt = settings.plotter_instance
+        plt = vedo.plotter_instance
+        n = int(self.fps * self.duration)
 
-        if zoom:
-            plt.camera.Zoom(zoom)
-
-        if isinstance(cam1, dict):
-            cam1 = buildcam(cam1)
-        if isinstance(cam2, dict):
-            cam2 = buildcam(cam2)
-
-        if len(elevation_range)==2:
-            plt.camera.Elevation(elevation_range[0])
-        if len(azimuth_range)==2:
-            plt.camera.Azimuth(azimuth_range[0])
+        cams = []
+        for cm in cameras:
+            cams.append(build_vtk_cam(cm))
+        nc = len(cams)
 
         plt.show(resetcam=resetcam, interactive=False)
-        # if resetcam: plt.renderer.ResetCamera()
 
-        n = self.fps * self.duration
-        for i in range(int(n)):
-            if cam1 and cam2:
-                plt.moveCamera(cam1, cam2, i/n)
-            else:
-                if len(elevation_range)==2:
-                    plt.camera.Elevation((elevation_range[1]-elevation_range[0])/n)
-                if len(azimuth_range)==2:
-                    plt.camera.Azimuth((azimuth_range[1]-azimuth_range[0])/n)
-            plt.show()
-            self.addFrame()
+        if nc:
+            for icam, cam in enumerate(cams):
+                if cam==cams[-1]:
+                    break
+                for i in range(n):
+                    plt.moveCamera(cams[icam], cams[icam+1], i/n)
+                    plt.show()
+                    self.addFrame()
+
+        else: ########################################
+
+            for i in range(n):
+                plt.camera.Elevation((elevation[1]-elevation[0])/n)
+                plt.camera.Azimuth((azimuth[1]-azimuth[0])/n)
+                plt.show()
+                self.addFrame()
+
         return self
 
+
     def close(self):
         """
         Render the video and write to file.
@@ -1912,21 +1895,21 @@ class Video:
                             + " -i " + self.tmp_dir.name + os.sep
                             + "%01d.png " + self.options + " " + self.name)
             if out:
-                colors.printc("ffmpeg returning error", c='r')
+                vedo.logger.error(f"backend {self.backend} returning error")
             else:
-                colors.printc("\save Video saved as", self.name, c="m")
+                colors.printc(f" \save video saved as {self.name}", c="m")
 
         ########################################
         elif 'cv' in self.backend:
             try:
                 import cv2
             except:
-                colors.printc("Error in Video backend: opencv not installed!", c='r')
+                vedo.logger.error("opencv is not installed")
                 return
 
             cap = cv2.VideoCapture(os.path.join(self.tmp_dir.name, "%1d.png"))
             fourcc = cv2.VideoWriter_fourcc(*"mp4v")
-            w,h = settings.plotter_instance.window.GetSize()
+            w,h = vedo.plotter_instance.window.GetSize()
             w,h = w*settings.screeshotScale, h*settings.screeshotScale
             writer = cv2.VideoWriter(self.name, fourcc, self.fps, (w, h), True)
 
@@ -1940,11 +1923,9 @@ class Video:
             cap.release()
             writer.release()
             if found:
-                colors.printc("\save Video saved as", self.name, c="m")
+                vedo.logger.info("video saved as {self.name}")
             else:
-                colors.printc("could not find snapshots", c='r')
+                vedo.logger.error("could not find snapshots")
 
         self.tmp_dir.cleanup()
-        return settings.plotter_instance
-
-
+        return vedo.plotter_instance
