@@ -1,11 +1,18 @@
-import numpy as np
 import os
-import vtk
+
+import numpy as np
 import vedo
-from vedo.colors import printc, getColor, colorMap
-from vedo.utils import isSequence, flatten, mag, buildPolyData, numpy2vtk, vtk2numpy
-from vedo.pointcloud import Points
+import vtk
 from deprecated import deprecated
+from vedo.colors import colorMap
+from vedo.colors import getColor
+from vedo.pointcloud import Points
+from vedo.utils import buildPolyData
+from vedo.utils import flatten
+from vedo.utils import isSequence
+from vedo.utils import mag
+from vedo.utils import numpy2vtk
+from vedo.utils import vtk2numpy
 
 __doc__ = ("""Submodule to manage polygonal meshes.""" + vedo.docs._defs)
 
@@ -41,7 +48,7 @@ def merge(*meshs, flag=False):
             poly = a
         polyapp.AddInputData(poly)
         if flag:
-            idarr += [i]*poly.GetNumberOfPoints()
+            idarr += [i] * poly.GetNumberOfPoints()
     polyapp.Update()
     mpoly = polyapp.GetOutput()
 
@@ -165,7 +172,7 @@ class Mesh(Points):
                 print("Could not add meshio cell data, skip.")
 
         elif "meshlab" in inputtype:
-            self._data = vedo.utils._meshlab2vedo(inputobj)
+            self._data = vedo.utils.meshlab2vedo(inputobj)
 
         elif isSequence(inputobj):
             ninp = len(inputobj)
@@ -201,13 +208,9 @@ class Mesh(Points):
                 gf.Update()
                 self._data = gf.GetOutput()
             except:
-                printc("Error: cannot build mesh from type:\n", inputtype, c='r')
+                vedo.logger.error(f"cannot build mesh from type {inputtype}")
                 raise RuntimeError()
 
-
-        if vedo.settings.computeNormals is not None:
-            computeNormals = vedo.settings.computeNormals
-
         if self._data:
             if computeNormals:
                 pdnorm = vtk.vtkPolyDataNormals()
@@ -396,8 +399,7 @@ class Mesh(Points):
             if not os.path.isfile(tname):
                 tname = str(self.filename).replace('.'+ext, '.jpg')
             if not os.path.isfile(tname):
-                printc("Error in texture(): default texture file must be png or jpg",
-                       "\n e.g.", tname, c='r')
+                vedo.logger.error("in texture() default texture file must be png or jpg")
                 raise RuntimeError()
 
         if isinstance(tname, vtk.vtkTexture):
@@ -407,13 +409,13 @@ class Mesh(Points):
                 if not isinstance(tcoords, np.ndarray):
                     tcoords = np.array(tcoords)
                 if tcoords.ndim != 2:
-                    printc('tcoords must be a 2-dimensional array', c='r')
+                    vedo.logger.error("tcoords must be a 2-dimensional array")
                     return self
                 if tcoords.shape[0] != pd.GetNumberOfPoints():
-                    printc('Error in texture(): nr of texture coords must match nr of points', c='r')
+                    vedo.logger.error("nr of texture coords must match nr of points")
                     return self
                 if tcoords.shape[1] != 2:
-                    printc('Error in texture(): vector must have 2 components', c='r')
+                    vedo.logger.error("vector must have 2 components")
                 tarr = numpy2vtk(tcoords)
                 tarr.SetName('TCoordinates')
                 pd.GetPointData().SetTCoords(tarr)
@@ -434,16 +436,11 @@ class Mesh(Points):
                     pd.GetPointData().SetTCoords(tc)
                     pd.GetPointData().Modified()
 
-            fn = vedo.settings.textures_path + tname + ".jpg"
+            fn = vedo.textures_path + tname + ".jpg"
             if os.path.exists(tname):
                 fn = tname
             elif not os.path.exists(fn):
-                printc("File does not exist or texture", tname,
-                       "not found in", vedo.settings.textures_path, c="r")
-                printc("\tin Available built-in textures:", c="m", end=" ")
-                for ff in os.listdir(vedo.settings.textures_path):
-                    printc(ff.split(".")[0], end=" ", c="m")
-                print()
+                vedo.logger.error(f"File does not exist or texture {tname} not found in {vedo.textures_path}")
                 return self
 
             fnl = fn.lower()
@@ -454,7 +451,7 @@ class Mesh(Points):
             elif ".bmp" in fnl:
                 reader = vtk.vtkBMPReader()
             else:
-                printc("Error in texture(): supported files, PNG, BMP or JPG", c="r")
+                vedo.logger.error("in texture(): supported files are only PNG, BMP or JPG")
                 return self
             reader.SetFileName(fn)
             reader.Update()
@@ -538,11 +535,19 @@ class Mesh(Points):
         Two flags are used to control these operations:
 
         - `cells=True` reverses the order of the indices in the cell connectivity list.
+        If cell is a list of IDs only those cells will be reversed.
 
         - `normals=True` reverses the normals by multiplying the normal vector by -1
             (both point and cell normals, if present).
         """
         poly = self.polydata(False)
+
+        if isSequence(cells):
+            for cell in cells:
+                poly.ReverseCell(cell)
+            poly.GetCellData().Modified()
+            return self ##############
+
         rev = vtk.vtkReverseSense()
         if cells:
             rev.ReverseCellsOn()
@@ -556,6 +561,7 @@ class Mesh(Points):
         rev.Update()
         return self._update(rev.GetOutput())
 
+
     def wireframe(self, value=True):
         """Set mesh's representation as wireframe or solid surface.
         Same as `mesh.wireframe()`."""
@@ -605,7 +611,6 @@ class Mesh(Points):
             return self
 
         if self.property.GetOpacity() < 1:
-            # printc("In backColor(): only active for alpha=1", c="y")
             return self
 
         if not backProp:
@@ -640,16 +645,12 @@ class Mesh(Points):
 
     def lineColor(self, lc=None):
         """Set/get color of mesh edges. Same as `lc()`."""
-        if lc is not None:
-#            if "ireframe" in self.property.GetRepresentationAsString():
-#                self.property.EdgeVisibilityOff()
-#                self.color(lc)
-#                return self
+        if lc is None:
+            return self.property.GetEdgeColor()
+        else:
             self.property.EdgeVisibilityOn()
             self.property.SetEdgeColor(getColor(lc))
-        else:
-            return self.property.GetEdgeColor()
-        return self
+            return self
 
     def lc(self, lineColor=None):
         """Set/get color of mesh edges. Same as `lineColor()`."""
@@ -703,6 +704,8 @@ class Mesh(Points):
         shrink.SetInputData(self._data)
         shrink.SetShrinkFactor(fraction)
         shrink.Update()
+        self.point_locator = None
+        self.cell_locator = None
         return self._update(shrink.GetOutput())
 
 
@@ -715,8 +718,7 @@ class Mesh(Points):
             two attributes ``mesh.base``, and ``mesh.top`` are already defined.
         """
         if self.base is None:
-            printc('Error in stretch(): Please define vectors', c='r')
-            printc('   mesh.base and mesh.top at creation.', c='r')
+            vedo.logger.error("in stretch() must define vectors mesh.base and mesh.top at creation")
             raise RuntimeError()
 
         p1, p2 = self.base, self.top
@@ -1002,43 +1004,9 @@ class Mesh(Points):
             return self._update(vct.GetOutput())
 
         else:
-            #printc("Error in triangulate()")
+            vedo.logger.debug("input in triangulate() seems to be void")
             return self
 
-    @deprecated(reason=vedo.colors.red+"Please use distanceTo()"+vedo.colors.reset)
-    def distanceToMesh(self, mesh, signed=False, negate=False):
-        return self.distanceTo(mesh, signed=signed, negate=negate)
-
-    def distanceTo(self, mesh, signed=False, negate=False, name="Distance"):
-        '''
-        Computes the (signed) distance from one mesh to another.
-
-        |distance2mesh| |distance2mesh.py|_
-        '''
-        poly1 = self.polydata()
-        poly2 = mesh.polydata()
-        df = vtk.vtkDistancePolyDataFilter()
-        df.ComputeSecondDistanceOff()
-        df.SetInputData(0, poly1)
-        df.SetInputData(1, poly2)
-        if signed:
-            df.SignedDistanceOn()
-        else:
-            df.SignedDistanceOff()
-        if negate:
-            df.NegateDistanceOn()
-        df.Update()
-
-        scals = df.GetOutput().GetPointData().GetScalars()
-        scals.SetName(name)
-        poly1.GetPointData().AddArray(scals)
-
-        poly1.GetPointData().SetActiveScalars(scals.GetName())
-        rng = scals.GetRange()
-        self._mapper.SetScalarRange(rng[0], rng[1])
-        self._mapper.ScalarVisibilityOn()
-        return self
-
     def addCellArea(self, name="Area"):
         """Add to this mesh a cell data array containing the areas of the polygonal faces"""
         csf = vtk.vtkCellSizeFilter()
@@ -1190,14 +1158,15 @@ class Mesh(Points):
         return self
 
 
-    def addShadow(self, plane=None, point=None, direction=None, clip=False, c=(0.6,0.6,0.6), alpha=1, culling=1):
+    def addShadow(self, plane=None, point=None, direction=None, clip=False,
+                  c=(0.6,0.6,0.6), alpha=1, culling=1):
         """
         Generate a shadow out of an ``Mesh`` on one of the three Cartesian planes.
         The output is a new ``Mesh`` representing the shadow.
         This new mesh is accessible through `mesh.shadow`.
         By default the shadow mesh is placed on the bottom wall of the bounding box.
 
-        See pointcloud.projectOnPlane.
+        See also pointcloud.projectOnPlane.
 
         :param str,Plane plane: if plane is `str`, plane can be one of ['x', 'y', 'z'],
             represents x-plane, y-plane and z-plane, respectively.
@@ -1217,26 +1186,41 @@ class Mesh(Points):
 
         |airplanes| |airplanes.py|_
         """
+        shad = self.clone()
+        pts = shad.points()
         if   'x' == plane:
-            shad = self.clone().projectOnPlane('x')
+            # shad = shad.projectOnPlane('x')
+            # instead do it manually so in case of alpha<1 we dont see glitches due to coplanar points
+            # we leave a small tolerance of 0.1% in thickness
+            x0,x1 = self.xbounds()
+            pts[:,0] = (pts[:,0]-(x0+x1)/2)/1000 + self.GetOrigin()[0]
+            shad.points(pts)
             if point is not None:
                 shad.x(point)
         elif 'y' == plane:
-            shad = self.clone().projectOnPlane('y')
+            # shad = shad.projectOnPlane('y')
+            x0,x1 = self.ybounds()
+            pts[:,1] = (pts[:,1]-(x0+x1)/2)/1000 + self.GetOrigin()[1]
+            shad.points(pts)
             if point is not None:
                 shad.y(point)
         elif 'z' == plane:
-            shad = self.clone().projectOnPlane('z')
+            # shad = shad.projectOnPlane('z')
+            x0,x1 = self.zbounds()
+            pts[:,2] = (pts[:,2]-(x0+x1)/2)/1000 + self.GetOrigin()[2]
+            shad.points(pts)
             if point is not None:
                 shad.z(point)
         else:
-            shad = self.clone().projectOnPlane(plane, point, direction, clip)
+            shad = shad.projectOnPlane(plane, point, direction, clip)
 
-        shad.c(c).alpha(alpha).wireframe(False).flat()
-        if culling==1:
+        shad.c(c).alpha(alpha).flat()
+
+        if culling==1 or culling==True:
             shad.frontFaceCulling()
         elif culling==-1:
             shad.backFaceCulling()
+
         shad.GetProperty().LightingOff()
         shad.SetPickable(False)
         shad.SetUseBounds(True)
@@ -1276,8 +1260,9 @@ class Mesh(Points):
         elif method == 3:
             sdf = vtk.vtkButterflySubdivisionFilter()
         else:
-            printc("Error in subdivide: unknown method.", c="r")
+            vedo.logger.error(f"in subdivide() unknown method {method}")
             raise RuntimeError()
+
         if method != 2:
             sdf.SetNumberOfSubdivisions(N)
         sdf.SetInputData(originalMesh)
@@ -1495,7 +1480,8 @@ class Mesh(Points):
             projection tolerance which controls how close the imprint surface must be to the target.
             The default is 0.01.
 
-        :Example:
+        Example
+        -------
 
             .. code-block:: python
 
@@ -1667,9 +1653,9 @@ class Mesh(Points):
             sil.SetFeatureAngle(featureAngle)
 
         if (direction is None
-            and vedo.settings.plotter_instance
-            and vedo.settings.plotter_instance.camera):
-            sil.SetCamera(vedo.settings.plotter_instance.camera)
+            and vedo.plotter_instance
+            and vedo.plotter_instance.camera):
+            sil.SetCamera(vedo.plotter_instance.camera)
             m = Mesh()
             m._mapper.SetInputConnection(sil.GetOutputPort())
 
@@ -1690,8 +1676,8 @@ class Mesh(Points):
             sil.Update()
             m = Mesh(sil.GetOutput())
         else:
-            printc('Error in silhouette(): direction is', [direction], c='r')
-            printc(' render the scene with show() or specify camera/direction', c='r')
+            vedo.logger.error(f"in silhouette() unknown direction type {type(direction)}")
+            vedo.logger.error("first render the scene with show() or specify camera/direction")
             return self
 
         m.lw(2).c((0,0,0)).lighting('off')
@@ -1713,7 +1699,7 @@ class Mesh(Points):
         if isinstance(cam, vtk.vtkCamera):
             self.SetCamera(cam)
         else:
-            plt = vedo.settings.plotter_instance
+            plt = vedo.plotter_instance
             if plt and plt.camera:
                 self.SetCamera(plt.camera)
             else:
@@ -1764,8 +1750,8 @@ class Mesh(Points):
         bcf = vtk.vtkBandedPolyDataContourFilter()
         bcf.SetInputData(self.polydata())
         # Use either the minimum or maximum value for each band.
-        for i in range(len(bands)):
-            bcf.SetValue(i, bands[i][2])
+        for i, band in enumerate(bands):
+            bcf.SetValue(i, band[2])
         # We will use an indexed lookup table.
         bcf.SetScalarModeToIndex()
         bcf.GenerateContourEdgesOff()
@@ -2033,8 +2019,111 @@ class Mesh(Points):
         prop.SetOpacity(1)
         dmesh.SetProperty(prop)
         dmesh.property = prop
-        dmesh.name = "geodesicLine"
+        dmesh.name = "GeodesicLine"
         return dmesh
 
+    #####################################################################
+    ### Stuff returning a Volume
+    ###
+    def binarize(self, spacing=(1,1,1), invert=False):
+        """
+        Convert a ``Mesh`` into a ``Volume``
+        where the foreground (exterior) voxels value is 255 and the background
+        (interior) voxels value is 0.
 
+        |mesh2volume| |mesh2volume.py|_
+        """
+        # https://vtk.org/Wiki/VTK/Examples/Cxx/PolyData/PolyDataToImageData
+        pd = self.polydata()
 
+        whiteImage = vtk.vtkImageData()
+        bounds = pd.GetBounds()
+
+        whiteImage.SetSpacing(spacing)
+
+        # compute dimensions
+        dim = [0, 0, 0]
+        for i in [0, 1, 2]:
+            dim[i] = int(np.ceil((bounds[i * 2 + 1] - bounds[i * 2]) / spacing[i]))
+        whiteImage.SetDimensions(dim)
+        whiteImage.SetExtent(0, dim[0] - 1, 0, dim[1] - 1, 0, dim[2] - 1)
+
+        origin = [0, 0, 0]
+        origin[0] = bounds[0] + spacing[0] / 2
+        origin[1] = bounds[2] + spacing[1] / 2
+        origin[2] = bounds[4] + spacing[2] / 2
+        whiteImage.SetOrigin(origin)
+        whiteImage.AllocateScalars(vtk.VTK_UNSIGNED_CHAR, 1)
+
+        # fill the image with foreground voxels:
+        if invert:
+            inval = 0
+        else:
+            inval = 255
+        count = whiteImage.GetNumberOfPoints()
+        for i in range(count):
+            whiteImage.GetPointData().GetScalars().SetTuple1(i, inval)
+
+        # polygonal data --> image stencil:
+        pol2stenc = vtk.vtkPolyDataToImageStencil()
+        pol2stenc.SetInputData(pd)
+        pol2stenc.SetOutputOrigin(origin)
+        pol2stenc.SetOutputSpacing(spacing)
+        pol2stenc.SetOutputWholeExtent(whiteImage.GetExtent())
+        pol2stenc.Update()
+
+        # cut the corresponding white image and set the background:
+        if invert:
+            outval = 255
+        else:
+            outval = 0
+        imgstenc = vtk.vtkImageStencil()
+        imgstenc.SetInputData(whiteImage)
+        imgstenc.SetStencilConnection(pol2stenc.GetOutputPort())
+        imgstenc.SetReverseStencil(invert)
+        imgstenc.SetBackgroundValue(outval)
+        imgstenc.Update()
+        return vedo.Volume(imgstenc.GetOutput())
+
+    def signedDistance(self, bounds=None, dims=(20,20,20), invert=False):
+        """
+        Compute the ``Volume`` object whose voxels contains the signed distance from
+        the mesh.
+
+        :param list bounds: bounds of the output volume.
+        :param list dims: dimensions (nr. of voxels) of the output volume.
+        :param bool invert: flip the sign.
+
+        See example script: |volumeFromMesh.py|_
+        """
+        if bounds is None:
+            bounds = self.GetBounds()
+        sx = (bounds[1]-bounds[0])/dims[0]
+        sy = (bounds[3]-bounds[2])/dims[1]
+        sz = (bounds[5]-bounds[4])/dims[2]
+
+        img = vtk.vtkImageData()
+        img.SetDimensions(dims)
+        img.SetSpacing(sx, sy, sz)
+        img.SetOrigin(bounds[0], bounds[2], bounds[4])
+        img.AllocateScalars(vtk.VTK_FLOAT, 1)
+
+        imp = vtk.vtkImplicitPolyDataDistance()
+        imp.SetInput(self.polydata())
+        b2 = bounds[2]
+        b4 = bounds[4]
+        d0, d1, d2 = dims
+
+        for i in range(d0):
+            x = i*sx+bounds[0]
+            for j in range(d1):
+                y = j*sy+b2
+                for k in range(d2):
+                    v = imp.EvaluateFunction((x, y, k*sz+b4))
+                    if invert:
+                        v = -v
+                    img.SetScalarComponentFromFloat(i,j,k, 0, v)
+
+        vol = vedo.Volume(img)
+        vol.name = "SignedVolume"
+        return vol
