@@ -1,13 +1,13 @@
-import vtk
 import numpy as np
 import vedo
-import vedo.settings as settings
-import vedo.utils as utils
+import vedo.addons as addons
 import vedo.colors as colors
 import vedo.shapes as shapes
-import vedo.addons as addons
+import vedo.utils as utils
+import vtk
 from vedo.assembly import Assembly
-from vedo.mesh import Mesh, merge
+from vedo.mesh import merge
+from vedo.mesh import Mesh
 from vedo.plotter import show # not used, but useful to import this
 
 __doc__ = """Plotting utility functions.""" + vedo.docs._defs
@@ -31,7 +31,6 @@ class Plot(Assembly):
     """
     Derived class of ``Assembly`` to manipulate plots.
     """
-
     def __init__(self, *objs):
 
         Assembly.__init__(self, *objs)
@@ -312,7 +311,7 @@ def plot(*args, **kwargs):
                 kwargs["mc"] = cc
                 break
         if opts:
-            colors.printc("Could not understand option(s):", opts, c="y")
+            vedo.logger.error(f"Could not understand options {opts}")
 
     if optidx == 1 or optidx is None:
         if utils.isSequence(args[0][0]):
@@ -658,6 +657,7 @@ def _plotxy(
     pad=0.05,
     axes={},
 ):
+
     line=False
     if lw>0:
         line=True
@@ -784,7 +784,7 @@ def _plotxy(
 
     if xerrors is not None and not errorBand:
         if len(xerrors) != len(data):
-            colors.printc("Error in plotxy(xerrors=...): mismatched array length.", c='r')
+            vedo.logger.error("in plotxy(xerrors=...): mismatch in array length")
             return None
         errs = []
         for i, dta in enumerate(data):
@@ -797,7 +797,7 @@ def _plotxy(
 
     if yerrors is not None and not errorBand:
         if len(yerrors) != len(data):
-            colors.printc("Error in plotxy(yerrors=...): mismatched array length.", c='r')
+            vedo.logger.error("in plotxy(yerrors=...): mismatch in array length")
             return None
         errs = []
         for i in range(len(data)):
@@ -845,6 +845,7 @@ def _plotxy(
         tit.pickable(False).z(3 * offs)
         acts.append(tit)
 
+
     if axes == 1 or axes == True:
         axes = {}
     if isinstance(axes, dict):  #####################
@@ -857,13 +858,13 @@ def _plotxy(
             ynew = utils.linInterpolate(tp[i], [0, 1], [y0lim, y1lim])
             # print(i, tp[i], ynew, ts[i])
             labs.append([ynew, ts[i]])
-        if "xtitle" not in axes: axes["xtitle"] = xtitle
-        if "ytitle" not in axes: axes["ytitle"] = ytitle
+        axes["xtitle"] = xtitle
+        axes["ytitle"] = ytitle
         axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
-        # axes["c"] = "k"
+        axes["c"] = c
         axes["yUseBounds"] = True
         axs = addons.Axes(**axes)
         axs.name = "axes"
@@ -872,8 +873,6 @@ def _plotxy(
         asse.SetOrigin(x0lim, y0lim, 0)
 
     else:
-#        settings.xtitle = xtitle
-#        settings.ytitle = ytitle
         asse = Plot(acts)
 
     asse.yscale = yscale
@@ -915,7 +914,7 @@ def _plotFxy(
             exec(code, namespace)
             z = namespace["zfunc"]
         except:
-            colors.printc("Syntax Error in _plotFxy()", c='r')
+            vedo.logger.error("Syntax Error in _plotFxy")
             return None
 
     if c is not None:
@@ -956,7 +955,7 @@ def _plotFxy(
         poly = cl.GetOutput()
 
     if not poly.GetNumberOfPoints():
-        colors.printc("Function is not real in the domain", c='r')
+        vedo.logger.error("function is not real in the domain")
         return None
 
     if zlim[0]:
@@ -1042,7 +1041,7 @@ def _plotFz(
             exec(code, namespace)
             z = namespace["zfunc"]
         except:
-            colors.printc("Syntax Error in complex plotFz()", c='r')
+            vedo.logger.error("Syntax Error in complex plotFz")
             return None
 
     ps = vtk.vtkPlaneSource()
@@ -1313,8 +1312,7 @@ def _barplot(
         cols = [c] * len(counts)
     else:
         m = "barplot error: data must be given as [counts, labels, colors, edges] not\n"
-        colors.printc(m, data, c='r')
-        colors.printc("     bin edges and colors are optional. Abort.", c='r')
+        vedo.logger.error(m + f" {data}\n     bin edges and colors are optional.")
         raise RuntimeError()
     counts = np.asarray(counts)
     edges  = np.asarray(edges)
@@ -1478,8 +1476,8 @@ def _barplot(
         for i in range(len(centers)):
             _xlabs.append([centers[i], str(xlabs[i])])
         axes["xValuesAndLabels"] = _xlabs
-        if "xtitle" not in axes: axes["xtitle"] = xtitle
-        if "ytitle" not in axes: axes["ytitle"] = ytitle
+        axes["xtitle"] = xtitle
+        axes["ytitle"] = ytitle
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
@@ -1490,8 +1488,6 @@ def _barplot(
         asse.axes = axs
         asse.SetOrigin(x0lim, y0lim, 0)
     else:
-#        settings.xtitle = xtitle
-#        settings.ytitle = ytitle
         asse = Plot(rs)
 
     asse.yscale = yscale
@@ -1649,7 +1645,7 @@ def _histogram1D(
                 col = c
             r.color(col).alpha(alpha).lighting('off').z(offs)
             rs.append(r)
-#            print('rectangles', r.z())
+            #print('rectangles', r.z())
 
     if outline:  #####################
         lns = [[myedges[0], 0, 0]]
@@ -1749,8 +1745,8 @@ def _histogram1D(
         for i in range(1, len(tp) - 1):
             ynew = utils.linInterpolate(tp[i], [0, 1], [y0lim, y1lim])
             labs.append([ynew, ts[i]])
-        if "xtitle" not in axes:  axes["xtitle"] = xtitle
-        if "ytitle" not in axes:  axes["ytitle"] = ytitle
+        axes["xtitle"] = xtitle
+        axes["ytitle"] = ytitle
         axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
@@ -1762,8 +1758,6 @@ def _histogram1D(
         asse.axes = axs
         asse.SetOrigin(x0lim, y0lim, 0)
     else:
-#        settings.xtitle = xtitle
-#        settings.ytitle = ytitle
         asse = Plot(rs)
 
     asse.yscale = yscale
@@ -1898,9 +1892,9 @@ def _histogram2D(
         for i in range(1, len(tp) - 1):
             ynew = utils.linInterpolate(tp[i], [0, 1], [y0lim, y1lim])
             labs.append([ynew, ts[i]])
-        if "xtitle" not in axes:  axes["xtitle"] = xtitle
-        if "ytitle" not in axes:  axes["ytitle"] = ytitle
-        if "ztitle" not in axes:  axes["ztitle"] = ztitle
+        axes["xtitle"] = xtitle
+        axes["ytitle"] = ytitle
+        axes["ztitle"] = ztitle
         axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
@@ -1912,9 +1906,6 @@ def _histogram2D(
         asse.axes = axs
         asse.SetOrigin(x0lim, y0lim, 0)
     else:
-#        settings.xtitle = xtitle
-#        settings.ytitle = ytitle
-#        settings.ytitle = ztitle
         asse = Plot(acts)
 
     asse.yscale = yscale
@@ -1936,26 +1927,19 @@ def _histogram2D(
 
 
 def _histogramHexBin(
-    xvalues,
-    yvalues,
-    xtitle="",
-    ytitle="",
-    ztitle="",
-    bins=12,
-    vrange=None,
-    norm=1,
-    fill=True,
-    c=None,
-    cmap="terrain_r",
-    alpha=1,
-):
-#    if xtitle:
-#        settings.xtitle = xtitle
-#    if ytitle:
-#        settings.ytitle = ytitle
-#    if ztitle:
-#        settings.ztitle = ztitle
-
+        xvalues,
+        yvalues,
+        xtitle="",
+        ytitle="",
+        ztitle="",
+        bins=12,
+        vrange=None,
+        norm=1,
+        fill=True,
+        c=None,
+        cmap="terrain_r",
+        alpha=1,
+    ):
     xmin, xmax = np.min(xvalues), np.max(xvalues)
     ymin, ymax = np.min(yvalues), np.max(yvalues)
     dx, dy = xmax - xmin, ymax - ymin
@@ -2731,7 +2715,7 @@ def matrix(M,
     return asse
 
 
-def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True, dots=True):
+def CornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True, dots=True):
     """
     Return a ``vtkXYPlotActor`` that is a plot of `x` versus `y`,
     where `points` is a list of `(x,y)` points.
@@ -2806,7 +2790,7 @@ def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True, dots=T
     return plot
 
 
-def cornerHistogram(
+def CornerHistogram(
     values,
     bins=20,
     vrange=None,
@@ -2856,13 +2840,13 @@ def cornerHistogram(
     for i in range(len(fs)):
         pts.append([(edges[i] + edges[i + 1]) / 2, fs[i]])
 
-    plot = cornerPlot(pts, pos, s, title, c, bg, lines, dots)
+    plot = CornerPlot(pts, pos, s, title, c, bg, lines, dots)
     plot.SetNumberOfYLabels(2)
     plot.SetNumberOfXLabels(3)
     tprop = vtk.vtkTextProperty()
     tprop.SetColor(colors.getColor(bg))
     tprop.SetFontFamily(vtk.VTK_FONT_FILE)
-    tprop.SetFontFile(utils.getFontPath(settings.defaultFont))
+    tprop.SetFontFile(utils.getFontPath(vedo.settings.defaultFont))
     tprop.SetOpacity(alpha)
     plot.SetAxisTitleTextProperty(tprop)
     plot.GetProperty().SetOpacity(alpha)
@@ -3032,15 +3016,14 @@ class DirectedGraph(Assembly):
             self.rotX = 180
 
         else:
-            colors.printc("Cannot understand layout:", s, c='r')
-            colors.printc("Available layouts:", c='r')
-            colors.printc("[2d,fast2d,clustering2d,circular,circular3d,cone,force,tree]", c='r')
+            vedo.logger.error(f"Cannot understand layout {s}. Available layouts:")
+            vedo.logger.error("[2d,fast2d,clustering2d,circular,circular3d,cone,force,tree]")
             raise RuntimeError()
 
         self.gl.SetLayoutStrategy(self.strategy)
 
         if len(kargs):
-            colors.printc("Cannot understand options:", kargs, c='r')
+            vedo.logger.error(f"Cannot understand options: {kargs}")
         return
 
 
