@@ -3,11 +3,6 @@ General settings.
 
 .. code-block:: python
 
-    # Axes title defaults
-    xtitle = 'x'
-    ytitle = 'y'
-    ztitle = 'z'
-
     # Set a default for the font to be used for axes, comments etc.
     defaultFont = 'Normografo' # check font options in shapes.Text
 
@@ -16,9 +11,6 @@ General settings.
     screenshotTransparentBackground = False
     screeshotLargeImage = False # Sometimes setting this to True gives better results
 
-    # Recompute vertex and cell normals
-    computeNormals = None
-
     # Allow to continously interact with scene during interactive() execution
     allowInteraction = False
 
@@ -34,7 +26,7 @@ General settings.
     rendererFrameColor = None
     rendererFrameAlpha = 0.5
     rendererFrameWidth = 0.5
-    rendererFramePadding = 0.0001
+    rendererFramePadding = 0.001
 
     # In multirendering mode set the position of the horizontal of vertical splitting [0,1]
     windowSplittingPosition = None
@@ -125,179 +117,140 @@ Usage example:
 
     Cube().color('green').show()
 """
-import os, vtk
-import numpy as np
-import warnings
 
-__all__ = ['dataurl', 'embedWindow']
+from vedo.utils import dotdict
+
+_setts = dotdict()
+_setts.warn_on_setting = False  # we are now initializing so disable warning
 
 
-vtk_version = [ int(vtk.vtkVersion().GetVTKMajorVersion()),
-                int(vtk.vtkVersion().GetVTKMinorVersion()),
-                int(vtk.vtkVersion().GetVTKBuildVersion()) ]
-
-try:
-    import platform
-    sys_platform = platform.system()
-except:
-    sys_platform = ""
-
-
-####################################################################################
-# Axes titles
-xtitle = 'x'
-ytitle = 'y'
-ztitle = 'z'
-
-# Set a default for the font to be used for axes, comments etc.
-defaultFont = 'Normografo'
+_setts.defaultFont = 'Normografo'
 
 # Scale magnification of the screenshot (must be an integer)
-screeshotScale = 1
-screenshotTransparentBackground = False
-screeshotLargeImage = False
-
-# Recompute vertex and cell normals
-computeNormals = None
+_setts.screeshotScale = 1
+_setts.screenshotTransparentBackground = False
+_setts.screeshotLargeImage = False
 
 # Allow to continously interact with scene during interactor.Start() execution
-allowInteraction = False
+_setts.allowInteraction = False
+
+# BUG in vtk9.0 (if true close works but sometimes vtk crashes, if false doesnt crash but cannot close)
+# see plotter.py line 555
+_setts.hackCallScreenSize = True
 
 # Set up default mouse and keyboard functionalities
-enableDefaultMouseCallbacks = True
-enableDefaultKeyboardCallbacks = True
+_setts.enableDefaultMouseCallbacks = True
+_setts.enableDefaultKeyboardCallbacks = True
 
 # When multiple renderers are present do not render each one for separate.
 # but do it just once at the end (when interactive() is called)
-immediateRendering = True
+_setts.immediateRendering = True
 
 # Show a gray frame margin in multirendering windows
-rendererFrameColor = None
-rendererFrameAlpha = 0.5
-rendererFrameWidth = 0.5
-rendererFramePadding = 0.0001
+_setts.rendererFrameColor = None
+_setts.rendererFrameAlpha = 0.5
+_setts.rendererFrameWidth = 0.5
+_setts.rendererFramePadding = 0.001
 
 # Wrap lines in tubes
 # renderPointsAsSpheres has become mesh.renderPointsAsSpheres(True)
-renderLinesAsTubes = False
+_setts.renderLinesAsTubes = False
 
 # Remove hidden lines when in wireframe mode
-hiddenLineRemoval = False
+_setts.hiddenLineRemoval = False
 
 # Smoothing options
-pointSmoothing = False
-lineSmoothing = False
-polygonSmoothing = False
+_setts.pointSmoothing = False
+_setts.lineSmoothing = False
+_setts.polygonSmoothing = False
 
 # For Structured and RectilinearGrid: show internal edges not only outline
-visibleGridEdges = False
+_setts.visibleGridEdges = False
 
 # Turn on/off the automatic repositioning of lights as the camera moves.
-lightFollowsCamera = False
-twoSidedLighting = True
+_setts.lightFollowsCamera = False
+_setts.twoSidedLighting = True
 
 # Turn on/off rendering of translucent material with depth peeling technique.
 #print("vtk_version sys_platform", vtk_version, sys_platform)
-useDepthPeeling = False
-multiSamples = 8
-if vtk_version[0] >= 9:
-    if "Windows" in sys_platform:
-        useDepthPeeling = True
+_setts.useDepthPeeling = False
+_setts.multiSamples = 8
+#if vtk_version[0] >= 9: # moved to __init__
+#    if "Windows" in sys_platform:
+#        useDepthPeeling = True
 # only relevant if depthpeeling is on
-alphaBitPlanes   = 1
-maxNumberOfPeels = 4
-occlusionRatio   = 0.1
+_setts.alphaBitPlanes   = 1
+_setts.maxNumberOfPeels = 4
+_setts.occlusionRatio   = 0.1
 
 # Turn on/off nvidia FXAA anti-aliasing, if supported.
-useFXAA = False  # either True or False
+_setts.useFXAA = False  # either True or False
 
 # By default, the depth buffer is reset for each renderer. If true, use the existing depth buffer
-preserveDepthBuffer = False
+_setts.preserveDepthBuffer = False
 
 #Enable or disable Screen Space Ambient Occlusion: SSAO darkens some pixels to improve depth perception.
-useSSAO        = False
-SSAORadius     = 0.5     # define the SSAO hemisphere radius
-SSAOBias       = 0.01    # define the bias when comparing samples
-SSAOKernelSize = 32      # define the number of samples
-SSAOBlur       = False   # define blurring of the ambient occlusion (helps for low samples)
+_setts.useSSAO        = False
+_setts.SSAORadius     = 0.5     # define the SSAO hemisphere radius
+_setts.SSAOBias       = 0.01    # define the bias when comparing samples
+_setts.SSAOKernelSize = 32      # define the number of samples
+_setts.SSAOBlur       = False   # define blurring of the ambient occlusion (helps for low samples)
 
 # Use a polygon/edges offset to possibly resolve conflicts in rendering
-usePolygonOffset = False
-polygonOffsetFactor = 0.1
-polygonOffsetUnits  = 0.1
+_setts.usePolygonOffset = False
+_setts.polygonOffsetFactor = 0.1
+_setts.polygonOffsetUnits  = 0.1
 
 # Interpolate scalars to render them smoothly
-interpolateScalarsBeforeMapping = True
+_setts.interpolateScalarsBeforeMapping = True
 
 # Set parallel projection On or Off (place camera to infinity, no perspective effects)
-useParallelProjection = False
+_setts.useParallelProjection = False
 
 # In multirendering mode set the position of the horizontal of vertical splitting [0,1]
-windowSplittingPosition = None
+_setts.windowSplittingPosition = None
 
 # Set orientation type when reading TIFF files (volumes):
 # TOPLEFT  1 (row 0 top, col 0 lhs)    TOPRIGHT 2 (row 0 top, col 0 rhs)
 # BOTRIGHT 3 (row 0 bottom, col 0 rhs) BOTLEFT  4 (row 0 bottom, col 0 lhs)
 # LEFTTOP  5 (row 0 lhs, col 0 top)    RIGHTTOP 6 (row 0 rhs, col 0 top)
 # RIGHTBOT 7 (row 0 rhs, col 0 bottom) LEFTBOT  8 (row 0 lhs, col 0 bottom)
-tiffOrientationType = 1
+_setts.tiffOrientationType = 1
 
 # AnnotatedCube axis (type 5) customization:
-annotatedCubeColor      = (0.75, 0.75, 0.75)
-annotatedCubeTextColor  = None # use default, otherwise specify a single color
-annotatedCubeTextScale  = 0.2
-annotatedCubeTexts      = ["right","left ", "front","back ", " top ", "bttom"]
+_setts.annotatedCubeColor      = (0.75, 0.75, 0.75)
+_setts.annotatedCubeTextColor  = None # use default, otherwise specify a single color
+_setts.annotatedCubeTextScale  = 0.2
+_setts.annotatedCubeTexts      = ["right","left ", "front","back ", " top ", "bttom"]
 
 # enable / disable color printing
-enablePrintColor = True
+_setts.enablePrintColor = True
 
 ####################################################################################
-# notebook support
-notebookBackend = None
-notebook_plotter = None
-
 # k3d settings for jupyter notebooks
-k3dMenuVisibility = True
-k3dPlotHeight = 512
-k3dAntialias  = True
-k3dLighting   = 1.2
-k3dCameraAutoFit = True
-k3dGridAutoFit= True
-k3dAxesHelper = True    # size of the small triad of axes on the bottom right
-k3dPointShader= "mesh"  # others are '3d', '3dSpecular', 'dot', 'flat'
-k3dLineShader = "thick" # others are 'flat', 'mesh'
+_setts.k3dMenuVisibility = True
+_setts.k3dPlotHeight = 512
+_setts.k3dAntialias  = True
+_setts.k3dLighting   = 1.2
+_setts.k3dCameraAutoFit = True
+_setts.k3dGridAutoFit= True
+_setts.k3dAxesHelper = True    # size of the small triad of axes on the bottom right
+_setts.k3dPointShader= "mesh"  # others are '3d', '3dSpecular', 'dot', 'flat'
+_setts.k3dLineShader = "thick" # others are 'flat', 'mesh'
 
 ####################################################################################
-flagDelay = 150 # values will be superseded
-flagFont = "Courier"
-flagFontSize = 18
-flagJustification = 0
-flagAngle = 0
-flagBold = False
-flagItalic = False
-flagShadow = False
-flagColor = 'k'
-flagBackgroundColor = 'w'
+_setts.flagDelay = 150 # values will be superseded
+_setts.flagFont = "Courier"
+_setts.flagFontSize = 18
+_setts.flagJustification = 0
+_setts.flagAngle = 0
+_setts.flagBold = False
+_setts.flagItalic = False
+_setts.flagShadow = False
+_setts.flagColor = 'k'
+_setts.flagBackgroundColor = 'w'
 
 
-#######################################################################################
-#######################################################################################
-installdir = os.path.dirname(__file__)
-
-textures_path = os.path.join(installdir, "textures/")
-textures = []
-
-fonts_path = os.path.join(installdir, "fonts/")
-fonts = []
-
-#dataurl = "/home/musy/Dropbox/Public/vktwork/vedo_data/"; print('\ndataurl=',dataurl)
-dataurl = "https://vedo.embl.es/examples/data/"
-
-plotter_instances = []
-plotter_instance = None
-
-interactorStyle = None # internal use only
-
 ####################################################################################
 ####################################################################################
 # mono       # means that all letters occupy the same space slot horizontally
@@ -305,7 +258,7 @@ interactorStyle = None # internal use only
 # lspacing   # horizontal spacing inbetween letters (not words)
 # islocal    # is locally stored in /fonts, otherwise it's on vedo.embl.es/fonts
 
-font_parameters = dict(
+_setts.font_parameters = dict(
 
         Normografo = dict(
                         mono = False,
@@ -475,7 +428,7 @@ font_parameters = dict(
                         dotsep = "·",
                         islocal = False,
                         ),
-        Meson= dict(
+        Meson = dict(
                         mono = False,
                         fscale = 0.8,
                         hspacing = 0.9,
@@ -483,7 +436,7 @@ font_parameters = dict(
                         dotsep = "~^.~ ",
                         islocal = False,
                         ),
-        Komika= dict(
+        Komika = dict(
                         mono = False,
                         fscale = 0.7,
                         hspacing = 0.75,
@@ -493,110 +446,9 @@ font_parameters = dict(
                         ),
 )
 
-
-####################################################################################
-def embedWindow(backend='ipyvtk', verbose=True):
-    """Use this function to control whether the rendering window is inside
-    the jupyter notebook or as an independent external window"""
-    global notebook_plotter, notebookBackend
-
-    if not backend:
-        notebookBackend = None
-        notebook_plotter = None
-        return
-    else:
-
-        if any(['SPYDER' in name for name in os.environ]):
-            notebookBackend = None
-            notebook_plotter = None
-            return
-
-        try:
-            get_ipython()
-        except:
-            notebookBackend = None
-            notebook_plotter = None
-            return
-
-    backend = backend.lower()
-    notebookBackend = backend
-
-    if backend=='k3d':
-        try:
-            import k3d
-            if k3d._version.version_info != (2, 7, 4):
-                print('Warning: only k3d version 2.7.4 is currently supported')
-                # print('> pip install k3d==2.7.4')
-
-        except:
-            notebookBackend = None
-            if verbose:
-                print('embedWindow(verbose=True): could not load k3d module, try:')
-                print('> pip install k3d==2.7.4')
-
-    elif 'ipygany' in backend: # ipygany
-        try:
-            import ipygany
-        except:
-            notebookBackend = None
-            if verbose:
-                print('embedWindow(verbose=True): could not load ipygany module, try:')
-                print('> pip install ipygany')
-
-    elif 'itk' in backend: # itkwidgets
-        try:
-            import itkwidgets
-        except:
-            notebookBackend = None
-            if verbose:
-                print('embedWindow(verbose=True): could not load itkwidgets module, try:')
-                print('> pip install itkwidgets')
-
-    elif backend.lower() == '2d':
-        pass
-
-    elif backend =='panel':
-        try:
-            import panel
-            panel.extension('vtk')
-        except:
-            if verbose:
-                print('embedWindow(verbose=True): could not load panel try:')
-                print('> pip install panel')
-
-    elif 'ipyvtk' in backend:
-        try:
-            from ipyvtklink.viewer import ViewInteractiveWidget
-        except:
-            if verbose:
-                print('embedWindow(verbose=True): could not load ipyvtklink try:')
-                print('> pip install ipyvtklink')
-
-    else:
-        print("Unknown backend", backend)
-        raise RuntimeError()
+###########################################################################
+# end of init so re-enable warning if trying to set a non existing setting
+_setts.warn_on_setting = True
+###########################################################################
 
 
-#####################
-def _init():
-    global plotter_instance, plotter_instances
-    global textures, fonts
-    global notebookBackend, notebook_plotter
-
-    plotter_instance = None
-    plotter_instances = []
-
-    for f in os.listdir(textures_path):
-        tfn = f.split(".")[0]
-        if 'earth' in tfn: continue
-        textures.append(tfn)
-
-    for f in os.listdir(fonts_path):
-        if '.npz' in f: continue
-        fonts.append(f.split(".")[0])
-    fonts = list(sorted(fonts))
-
-    warnings.simplefilter(action="ignore", category=FutureWarning)
-    np.warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning)
-
-    embedWindow()
