@@ -1,9 +1,10 @@
 import numpy as np
-import vtk
-from vedo.base import BaseGrid
+import vedo
 import vedo.colors as colors
-import vedo.settings as settings
 import vedo.utils as utils
+import vtk
+from vedo import settings
+from vedo.base import BaseGrid
 
 __all__ = ["UGrid"]
 
@@ -91,12 +92,11 @@ class UGrid(vtk.vtkActor, BaseGrid):
             self.filename = inputobj
 
         else:
-            colors.printc("UGrid(): cannot understand input type:\n", inputtype, c='r')
+            vedo.logger.error(f"cannot understand input type {inputtype}")
             return
 
         # self._mapper = vtk.vtkDataSetMapper()
         self._mapper = vtk.vtkPolyDataMapper()
-
         self._mapper.SetInterpolateScalarsBeforeMapping(settings.interpolateScalarsBeforeMapping)
 
         if settings.usePolygonOffset:
